/***************************************************************************
                          crystalsymop.h  -  description
                             -------------------
    begin                : Mon Sept 28 07:53:00 CEST 2020
    copyright            : (C) 2020 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CRYSTALSYMOP_H
#define CRYSTALSYMOP_H

#include <QStringList>
#include <QString>

class CrystalAtom;

/*
 * /m11 m12 m13\ /m14\
 * |m21 m22 m23| |m24|
 * \m31 m32 m33/ \m34/
 */

#if defined XRDIO
 #define XRDIO_EXPORT Q_DECL_EXPORT
#else
 #define XRDIO_EXPORT Q_DECL_IMPORT
#endif

class XRDIO_EXPORT CrystalSymOp
{
public:
    CrystalSymOp(float _x11, float _x12, float _x13, float _x14,
                 float _x21, float _x22, float _x23, float _x24,
                 float _x31, float _x32, float _x33, float _x34);

    CrystalSymOp(const CrystalSymOp &);
    CrystalSymOp(const QStringList &);
    CrystalSymOp(const QString &);

    void operator=(const CrystalSymOp &);
    bool operator==(const CrystalSymOp &) const;

    void setMatrix(float _x11, float _x12, float _x13, float _x14,
                   float _x21, float _x22, float _x23, float _x24,
                   float _x31, float _x32, float _x33, float _x34);
    void setSymOp(const QStringList &);
    void setSymOp(const QString &);

    bool isEmpty() const;

    CrystalAtom transform(const CrystalAtom &);
    QList<CrystalSymOp> translate(const QString &);
    QStringList toStringList() const;

    inline float m11() const {return _m11;}
    inline float m12() const {return _m12;}
    inline float m13() const {return _m13;}
    inline float m14() const {return _m14;}
    inline float m21() const {return _m21;}
    inline float m22() const {return _m22;}
    inline float m23() const {return _m23;}
    inline float m24() const {return _m24;}
    inline float m31() const {return _m31;}
    inline float m32() const {return _m32;}
    inline float m33() const {return _m33;}
    inline float m34() const {return _m34;}

private:
    float _m11;
    float _m12;
    float _m13;
    float _m14;
    float _m21;
    float _m22;
    float _m23;
    float _m24;
    float _m31;
    float _m32;
    float _m33;
    float _m34;

    void parseString(const QStringList &);
    void parseOperator(const QString &, float &, float &, float &, float &);
};

#endif // CRYSTALSYMOP_H
