/***************************************************************************
                          curvefittingmanager.h  -  description
                             -------------------
    begin                : Tue Nov 12 19:20:03 CEST 2019
    copyright            : (C) 2019 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CURVEFITTINGMANAGER_H
#define CURVEFITTINGMANAGER_H

#include <QtCore/QtGlobal>
#include "genericcurve.h"
#include "../scan.h"
#include "../3rdparty/alglib/src/interpolation.h"

#if defined XRDIO
 #define XRDIO_EXPORT Q_DECL_EXPORT
#else
 #define XRDIO_EXPORT Q_DECL_IMPORT
#endif

class XRDIO_EXPORT CurveFittingManager
{
public:
    CurveFittingManager();

    void setControls(alglib::ae_int_t m, double e, double d);
    void setScanData(const Scan &_scan);
    void setCurves(const QList<GenericCurve *> &_clist);
    void setValues(const QVector<double> &_initVal, const QVector<double> &_loLimits, const QVector<double> &_upLimits);
    QVector<double> fit();
    inline QVector<double> getValues() const {return initValues;}
    inline QMap<QString, QVariant> getReport() const {return fitReport;}
    QList<Scan> calculateCurve(double stepSize, const QString &sumName, const QUuid &sumUid);
    double getArea(int, QUuid &, bool &ok);
    inline int numCurves() const {return curvesList().size();}

private:
    Scan dataRange;
    QVector<double> initValues;
    QVector<double> lowerLimits;
    QVector<double> upperLimits;
    QMap<QString, QVariant> fitReport;
    alglib::ae_int_t maxits;
    double epsx;
    double diffstep;
    int iteration;

    static QList<GenericCurve *> & curvesList();
    static void fitCurve(const alglib::real_1d_array &c, const alglib::real_1d_array &x, double &func, void *);
    static void stepOutput(const alglib::real_1d_array &c, double func, void *);
    QMap<QString, QVariant> generateReportString(const QList<GenericCurve*> &, const alglib::ae_int_t &, const alglib::lsfitreport &);
    QVector<double> getXvalues(double);
};


#endif // CURVEFITTINGMANAGER_H
