/***************************************************************************
                          curvehandler.cpp  -  description
                             -------------------
    begin                : Thu Jan 16 19:20:03 CEST 2020
    copyright            : (C) 2020 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "curvehandler.h"

CurveHandler::CurveHandler()
{

}

GenericCurve * CurveHandler::createCurve(int type)
{
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LINEAR)           return new LinearCurve;
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::GAUSSIAN)         return new GaussianCurve;
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LORENTZIAN)       return new LorentzianCurve;
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PSEUDOVOIGT)      return new PseudoVoigtCurve;
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PEARSONVII)       return new PearsonCurve;

    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::GAUSSIANSPLIT)    return new GaussianSplitCurve;
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LORENTZIANSPLIT)  return new LorentzianSplitCurve;
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PSEUDOVOIGTSPLIT) return new PseudoVoigtSplitCurve;
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PEARSONSPLIT)     return new PearsonSplitCurve;

    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::QUADRATIC)        return new QuadraticCurve;
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::CUBIC)            return new CubicCurve;
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::POLY4)            return new Polynom4Curve;
    return new GenericCurve;
}

QString CurveHandler::description(int type)
{
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LINEAR)           return LinearCurve::descriptionStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::GAUSSIAN)         return GaussianCurve::descriptionStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LORENTZIAN)       return LorentzianCurve::descriptionStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PSEUDOVOIGT)      return PseudoVoigtCurve::descriptionStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PEARSONVII)       return PearsonCurve::descriptionStatic();

    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::GAUSSIANSPLIT)    return GaussianSplitCurve::descriptionStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LORENTZIANSPLIT)  return LorentzianSplitCurve::descriptionStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PSEUDOVOIGTSPLIT) return PseudoVoigtSplitCurve::descriptionStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PEARSONSPLIT)     return PearsonSplitCurve::descriptionStatic();

    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::QUADRATIC)        return QuadraticCurve::descriptionStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::CUBIC)            return CubicCurve::descriptionStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::POLY4)            return Polynom4Curve::descriptionStatic();
    return GenericCurve::descriptionStatic();
}

QString CurveHandler::equationText(int type)
{
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LINEAR)           return LinearCurve::equationTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::GAUSSIAN)         return GaussianCurve::equationTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LORENTZIAN)       return LorentzianCurve::equationTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PSEUDOVOIGT)      return PseudoVoigtCurve::equationTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PEARSONVII)       return PearsonCurve::equationTextStatic();

    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::GAUSSIANSPLIT)    return GaussianSplitCurve::equationTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LORENTZIANSPLIT)  return LorentzianSplitCurve::equationTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PSEUDOVOIGTSPLIT) return PseudoVoigtSplitCurve::equationTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PEARSONSPLIT)     return PearsonSplitCurve::equationTextStatic();

    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::QUADRATIC)        return QuadraticCurve::equationTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::CUBIC)            return CubicCurve::equationTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::POLY4)            return Polynom4Curve::equationTextStatic();
    return GenericCurve::equationTextStatic();
}

QStringList CurveHandler::parameterText(int type)
{
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LINEAR)           return LinearCurve::parameterTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::GAUSSIAN)         return GaussianCurve::parameterTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LORENTZIAN)       return LorentzianCurve::parameterTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PSEUDOVOIGT)      return PseudoVoigtCurve::parameterTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PEARSONVII)       return PearsonCurve::parameterTextStatic();

    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::GAUSSIANSPLIT)    return GaussianSplitCurve::parameterTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::LORENTZIANSPLIT)  return LorentzianSplitCurve::parameterTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PSEUDOVOIGTSPLIT) return PseudoVoigtSplitCurve::parameterTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::PEARSONSPLIT)     return PearsonSplitCurve::parameterTextStatic();

    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::QUADRATIC)        return QuadraticCurve::parameterTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::CUBIC)            return CubicCurve::parameterTextStatic();
    if (static_cast<GenericCurve::CurveType>(type) == GenericCurve::POLY4)            return Polynom4Curve::parameterTextStatic();
    return GenericCurve::parameterTextStatic();
}
