/***************************************************************************
                          bgmndiaimport.h  -  description
                             -------------------
    begin                : Thu Jan 26, 2012
    copyright            : (C) 2012 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BGMNDIAIMPORT_H
#define BGMNDIAIMPORT_H

#include "genericimport.h"
#include "../hkl.h"

#include <QStringList>
#include <QColor>
#include <QRegularExpression>

#if defined XRDIO
 #define XRDIO_EXPORT Q_DECL_EXPORT
#else
 #define XRDIO_EXPORT Q_DECL_IMPORT
#endif

class XRDIO_EXPORT BgmnDiaImport : public GenericImport
{
    Q_OBJECT

    public:
        explicit BgmnDiaImport(QObject *parent = 0);

        bool isSupported(const QByteArray &);
        int load(const QString &, QVector<Scan> &, bool minimal = false);
        int loadAndCheck(const QString &, QVector<Scan> &, int &ll, int &tl, bool minimal = false);
        QString uniqueId() {return "BGMN_DIA";}

    private:
        QByteArray _header;
        QList<QVector<double> > _data;
        int _multiScans;
        bool _hasValidContent;
        bool _addBackground;

        int checkSize(int &ll, int &tl);
        int createScans(QVector<Scan> &, const QString &);
        double getWaveLength(const QByteArray &);
        QStringList getStructureNames(const QByteArray &);
        bool parseBinaryData(QList<QVector<double> > &, const QList<QByteArray> &);
        static QVector<double> parseLine(const QByteArray &);
        int countMultiScans(const QList<QByteArray> &) const;
        QList<QVector<double> > mergeMultiScans(const QList<QVector<double> > &, int);

    signals:
        void maxSteps(int);
        void progress(int);

};

#endif // BGMNDIAIMPORT_H
