/***************************************************************************
                          bgmnsgdatparser.h  -  description
                             -------------------
    begin                : Thu May 17 19:10:00 CEST 2018
    copyright            : (C) 2018 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BGMNSGDATPARSER_H
#define BGMNSGDATPARSER_H

#include <QObject>
#include <QMap>
#include <QList>
#include <QDomDocument>
#include "../structs.h"

#if defined XRDIO
 #define XRDIO_EXPORT Q_DECL_EXPORT
#else
 #define XRDIO_EXPORT Q_DECL_IMPORT
#endif

struct BgmnSpaceGroup {
    int itNumber;
    int setting;
    QString HermannMauguin;
    QString lattice;
    QString uniqueAxis;
    int cellChoice;
    int originChoice;
    QString settingLine;
};

struct ItNumSetting {
    int itNum;
    int setting;
    ItNumSetting() : itNum(), setting() {}
    ItNumSetting(int n, int s) : itNum(n), setting(s) {}
};

class XRDIO_EXPORT BgmnSgDatParser
{
public:
    explicit BgmnSgDatParser();

    void init();
    QList<BgmnSpaceGroup> getSpaceGroup(const QString &hm, int it = 0);
    inline const QMap<int, QMap<int, BgmnSpaceGroup> > & getAllSpaceGroups() const {return sgMap;}
    QMap<int, QList<global::WyckoffPosition> > getAllWyckoff(int it);
    QList<global::WyckoffPosition> getAllWyckoff(int it, int setting);
    QMap<QString, int> getAllMultiplicities(int it, int setting);
    QList<int> getSpaceGroupSettings(int it);
    QList<int> getSpaceGroupSettings(int it, const QString &transl);
    QList<QStringList> getSymmetryOperations(int it, int setting);

private:
    // Format: <itNumber <setting,  spacegroup> >
    QMap<int, QMap<int, BgmnSpaceGroup> > sgMap;
    QMap<int, QMap<int, QList<QStringList> > > sgSymOps;
    QDomDocument spaceGroupDat;

    // short HM symbols
    const QMap<QString, QList<ItNumSetting> > hmShort {
        {"A-1", {{ItNumSetting(2,3)}}},
        {"A1", {{ItNumSetting(1,3)}}},
        {"A2", {{ItNumSetting(5,2)}, {ItNumSetting(5,3)}, {ItNumSetting(5,7)}, {ItNumSetting(5,10)}}},
        {"A2/a", {{ItNumSetting(15,2)}, {ItNumSetting(15,3)}}},
        {"A2/m", {{ItNumSetting(12,2)}, {ItNumSetting(12,3)}, {ItNumSetting(12,7)}, {ItNumSetting(12,10)}}},
        {"A2/n", {{ItNumSetting(15,7)}, {ItNumSetting(15,10)}}},
        {"A2122", {{ItNumSetting(20,3)}, {ItNumSetting(20,4)}}},
        {"A21am", {{ItNumSetting(36,4)}}},
        {"A21ma", {{ItNumSetting(36,3)}}},
        {"A222", {{ItNumSetting(21,3)}, {ItNumSetting(21,4)}}},
        {"A2aa", {{ItNumSetting(37,3)}, {ItNumSetting(37,4)}}},
        {"A2mm", {{ItNumSetting(35,3)}, {ItNumSetting(35,4)}}},
        {"Aa", {{ItNumSetting(9,2)}, {ItNumSetting(9,3)}}},
        {"Aba2", {{ItNumSetting(41,1)}}},
        {"Abaa", {{ItNumSetting(68,5)}, {ItNumSetting(68,6)}}},
        {"Abm2", {{ItNumSetting(39,1)}}},
        {"Abma", {{ItNumSetting(64,3)}}},
        {"Abmm", {{ItNumSetting(67,3)}}},
        {"Ac2a", {{ItNumSetting(41,6)}}},
        {"Ac2m", {{ItNumSetting(39,6)}}},
        {"Acaa", {{ItNumSetting(68,7)}, {ItNumSetting(68,8)}}},
        {"Acam", {{ItNumSetting(64,4)}}},
        {"Acmm", {{ItNumSetting(67,4)}}},
        {"Am", {{ItNumSetting(8,2)}, {ItNumSetting(8,3)}, {ItNumSetting(8,7)}, {ItNumSetting(8,10)}}},
        {"Am2a", {{ItNumSetting(40,6)}}},
        {"Am2m", {{ItNumSetting(38,6)}}},
        {"Ama2", {{ItNumSetting(40,1)}}},
        {"Amaa", {{ItNumSetting(66,3)}, {ItNumSetting(66,4)}}},
        {"Amam", {{ItNumSetting(63,4)}}},
        {"Amm2", {{ItNumSetting(38,1)}}},
        {"Amma", {{ItNumSetting(63,3)}}},
        {"Ammm", {{ItNumSetting(65,3)}, {ItNumSetting(65,4)}}},
        {"An", {{ItNumSetting(9,7)}, {ItNumSetting(9,10)}}},
        {"B-1", {{ItNumSetting(2,4)}}},
        {"B1", {{ItNumSetting(1,4)}}},
        {"B2", {{ItNumSetting(5,4)}, {ItNumSetting(5,5)}, {ItNumSetting(5,9)}, {ItNumSetting(5,12)}}},
        {"B2/b", {{ItNumSetting(15,4)}, {ItNumSetting(15,5)}}},
        {"B2/m", {{ItNumSetting(12,4)}, {ItNumSetting(12,5)}, {ItNumSetting(12,9)}, {ItNumSetting(12,12)}}},
        {"B2/n", {{ItNumSetting(15,9)}, {ItNumSetting(15,12)}}},
        {"B2212", {{ItNumSetting(20,5)}, {ItNumSetting(20,6)}}},
        {"B222", {{ItNumSetting(21,5)}, {ItNumSetting(21,6)}}},
        {"B2cb", {{ItNumSetting(41,3)}}},
        {"B2cm", {{ItNumSetting(39,3)}}},
        {"B2mb", {{ItNumSetting(40,3)}}},
        {"B2mm", {{ItNumSetting(38,3)}}},
        {"Bb", {{ItNumSetting(9,4)}, {ItNumSetting(9,5)}}},
        {"Bb21m", {{ItNumSetting(36,5)}}},
        {"Bb2b", {{ItNumSetting(37,5)}, {ItNumSetting(37,6)}}},
        {"Bba2", {{ItNumSetting(41,2)}}},
        {"Bbab", {{ItNumSetting(68,11)}, {ItNumSetting(68,12)}}},
        {"Bbcb", {{ItNumSetting(68,9)}, {ItNumSetting(68,10)}}},
        {"Bbcm", {{ItNumSetting(64,5)}}},
        {"Bbm2", {{ItNumSetting(40,2)}}},
        {"Bbmb", {{ItNumSetting(66,5)}, {ItNumSetting(66,6)}}},
        {"Bbmm", {{ItNumSetting(63,5)}}},
        {"Bm", {{ItNumSetting(8,4)}, {ItNumSetting(8,5)}, {ItNumSetting(8,9)}, {ItNumSetting(8,12)}}},
        {"Bm21b", {{ItNumSetting(36,6)}}},
        {"Bm2m", {{ItNumSetting(35,5)}, {ItNumSetting(35,6)}}},
        {"Bma2", {{ItNumSetting(39,2)}}},
        {"Bmab", {{ItNumSetting(64,6)}}},
        {"Bmam", {{ItNumSetting(67,6)}}},
        {"Bmcm", {{ItNumSetting(67,5)}}},
        {"Bmm2", {{ItNumSetting(38,2)}}},
        {"Bmmb", {{ItNumSetting(63,6)}}},
        {"Bmmm", {{ItNumSetting(65,5)}, {ItNumSetting(65,6)}}},
        {"Bn", {{ItNumSetting(9,9)}, {ItNumSetting(9,12)}}},
        {"C-1", {{ItNumSetting(2,2)}}},
        {"C1", {{ItNumSetting(1,2)}}},
        {"C2", {{ItNumSetting(5,1)}, {ItNumSetting(5,6)}, {ItNumSetting(5,8)}, {ItNumSetting(5,11)}}},
        {"C2/c", {{ItNumSetting(15,1)}, {ItNumSetting(15,6)}}},
        {"C2/m", {{ItNumSetting(12,1)}, {ItNumSetting(12,6)}, {ItNumSetting(12,8)}, {ItNumSetting(12,11)}}},
        {"C2/n", {{ItNumSetting(15,8)}, {ItNumSetting(15,11)}}},
        {"C222", {{ItNumSetting(21,1)}, {ItNumSetting(21,2)}}},
        {"C2221", {{ItNumSetting(20,1)}, {ItNumSetting(20,2)}}},
        {"C2cb", {{ItNumSetting(41,4)}}},
        {"C2cm", {{ItNumSetting(40,4)}}},
        {"C2mb", {{ItNumSetting(39,4)}}},
        {"C2mm", {{ItNumSetting(38,4)}}},
        {"Cc", {{ItNumSetting(9,1)}, {ItNumSetting(9,6)}}},
        {"Cc2a", {{ItNumSetting(41,5)}}},
        {"Cc2m", {{ItNumSetting(40,5)}}},
        {"Ccc2", {{ItNumSetting(37,1)}, {ItNumSetting(37,2)}}},
        {"Ccca", {{ItNumSetting(68,1)}, {ItNumSetting(68,2)}}},
        {"Cccb", {{ItNumSetting(68,3)}, {ItNumSetting(68,4)}}},
        {"Cccm", {{ItNumSetting(66,1)}, {ItNumSetting(66,2)}}},
        {"Ccm21", {{ItNumSetting(36,2)}}},
        {"Ccmb", {{ItNumSetting(64,2)}}},
        {"Ccmm", {{ItNumSetting(63,2)}}},
        {"Cm", {{ItNumSetting(8,1)}, {ItNumSetting(8,6)}, {ItNumSetting(8,8)}, {ItNumSetting(8,11)}}},
        {"Cm2a", {{ItNumSetting(39,5)}}},
        {"Cm2m", {{ItNumSetting(38,5)}}},
        {"Cmc21", {{ItNumSetting(36,1)}}},
        {"Cmca", {{ItNumSetting(64,1)}}},
        {"Cmcm", {{ItNumSetting(63,1)}}},
        {"Cmm2", {{ItNumSetting(35,1)}, {ItNumSetting(35,2)}}},
        {"Cmma", {{ItNumSetting(67,1)}}},
        {"Cmmb", {{ItNumSetting(67,2)}}},
        {"Cmmm", {{ItNumSetting(65,1)}, {ItNumSetting(65,2)}}},
        {"Cn", {{ItNumSetting(9,8)}, {ItNumSetting(9,11)}}},
        {"F-1", {{ItNumSetting(2,6)}}},
        {"F-43c", {{ItNumSetting(219,1)}}},
        {"F-43m", {{ItNumSetting(216,1)}}},
        {"F1", {{ItNumSetting(1,6)}}},
        {"F2", {{ItNumSetting(3,7)}}},
        {"F222", {{ItNumSetting(22,1)}, {ItNumSetting(22,2)}, {ItNumSetting(22,3)}, {ItNumSetting(22,4)}, {ItNumSetting(22,5)}, {ItNumSetting(22,6)}}},
        {"F23", {{ItNumSetting(196,1)}}},
        {"F2dd", {{ItNumSetting(43,3)}, {ItNumSetting(43,4)}}},
        {"F2mm", {{ItNumSetting(42,3)}, {ItNumSetting(42,4)}}},
        {"F4132", {{ItNumSetting(210,1)}}},
        {"F432", {{ItNumSetting(209,1)}}},
        {"Fd-3", {{ItNumSetting(203,1)}, {ItNumSetting(203,2)}}},
        {"Fd-3c", {{ItNumSetting(228,1)}, {ItNumSetting(228,2)}}},
        {"Fd-3m", {{ItNumSetting(227,1)}, {ItNumSetting(227,2)}}},
        {"Fd3m", {{ItNumSetting(227,1)}, {ItNumSetting(227,2)}}}, // this is a wrong symbol, but it seems to be used a lot
        {"Fd2d", {{ItNumSetting(43,5)}, {ItNumSetting(43,6)}}},
        {"Fdd2", {{ItNumSetting(43,1)}, {ItNumSetting(43,2)}}},
        {"Fddd", {{ItNumSetting(70,1)}, {ItNumSetting(70,2)}, {ItNumSetting(70,3)}, {ItNumSetting(70,4)}, {ItNumSetting(70,5)}, {ItNumSetting(70,6)}, {ItNumSetting(70,7)}, {ItNumSetting(70,8)}, {ItNumSetting(70,9)}, {ItNumSetting(70,10)}, {ItNumSetting(70,11)}, {ItNumSetting(70,12)}}},
        {"Fm-3", {{ItNumSetting(202,1)}}},
        {"Fm-3c", {{ItNumSetting(226,1)}}},
        {"Fm-3m", {{ItNumSetting(225,1)}}},
        {"Fm3m", {{ItNumSetting(225,1)}}}, // this is a wrong symbol, but it seems to be used a lot
        {"Fm2m", {{ItNumSetting(42,5)}, {ItNumSetting(42,6)}}},
        {"Fmm2", {{ItNumSetting(42,1)}, {ItNumSetting(42,2)}}},
        {"Fmmm", {{ItNumSetting(69,1)}, {ItNumSetting(69,2)}, {ItNumSetting(69,3)}, {ItNumSetting(69,4)}, {ItNumSetting(69,5)}, {ItNumSetting(69,6)}}},
        {"I-1", {{ItNumSetting(2,5)}}},
        {"I-4", {{ItNumSetting(82,1)}}},
        {"I-42d", {{ItNumSetting(122,1)}}},
        {"I-42m", {{ItNumSetting(121,1)}}},
        {"I-43d", {{ItNumSetting(220,1)}}},
        {"I-43m", {{ItNumSetting(217,1)}}},
        {"I-4c2", {{ItNumSetting(120,1)}}},
        {"I-4m2", {{ItNumSetting(119,1)}}},
        {"I1", {{ItNumSetting(1,5)}}},
        {"I2", {{ItNumSetting(5,13)}, {ItNumSetting(5,14)}, {ItNumSetting(5,15)}, {ItNumSetting(5,16)}, {ItNumSetting(5,17)}, {ItNumSetting(5,18)}}},
        {"I2/a", {{ItNumSetting(15,13)}, {ItNumSetting(15,16)}}},
        {"I2/b", {{ItNumSetting(15,15)}, {ItNumSetting(15,18)}}},
        {"I2/c", {{ItNumSetting(15,14)}, {ItNumSetting(15,17)}}},
        {"I2/m", {{ItNumSetting(12,13)}, {ItNumSetting(12,14)}, {ItNumSetting(12,15)}, {ItNumSetting(12,16)}, {ItNumSetting(12,17)}, {ItNumSetting(12,18)}}},
        {"I212121", {{ItNumSetting(24,1)}, {ItNumSetting(24,2)}, {ItNumSetting(24,3)}, {ItNumSetting(24,4)}, {ItNumSetting(24,5)}, {ItNumSetting(24,6)}}},
        {"I213", {{ItNumSetting(199,1)}}},
        {"I222", {{ItNumSetting(23,1)}, {ItNumSetting(23,2)}, {ItNumSetting(23,3)}, {ItNumSetting(23,4)}, {ItNumSetting(23,5)}, {ItNumSetting(23,6)}}},
        {"I23", {{ItNumSetting(197,1)}}},
        {"I2cb", {{ItNumSetting(45,3)}, {ItNumSetting(45,4)}}},
        {"I2cm", {{ItNumSetting(46,4)}}},
        {"I2mb", {{ItNumSetting(46,3)}}},
        {"I2mm", {{ItNumSetting(44,3)}, {ItNumSetting(44,4)}}},
        {"I4", {{ItNumSetting(79,1)}}},
        {"I4/m", {{ItNumSetting(87,1)}}},
        {"I4/mcm", {{ItNumSetting(140,1)}}},
        {"I4/mmm", {{ItNumSetting(139,1)}}},
        {"I41", {{ItNumSetting(80,1)}}},
        {"I41/a", {{ItNumSetting(88,1)}, {ItNumSetting(88,2)}}},
        {"I41/acd", {{ItNumSetting(142,1)}, {ItNumSetting(142,2)}}},
        {"I41/amd", {{ItNumSetting(141,1)}, {ItNumSetting(141,2)}}},
        {"I4122", {{ItNumSetting(98,1)}}},
        {"I4132", {{ItNumSetting(214,1)}}},
        {"I41cd", {{ItNumSetting(110,1)}}},
        {"I41md", {{ItNumSetting(109,1)}}},
        {"I422", {{ItNumSetting(97,1)}}},
        {"I432", {{ItNumSetting(211,1)}}},
        {"I4cm", {{ItNumSetting(108,1)}}},
        {"I4mm", {{ItNumSetting(107,1)}}},
        {"Ia", {{ItNumSetting(9,13)}, {ItNumSetting(9,16)}}},
        {"Ia-3", {{ItNumSetting(206,1)}}},
        {"Ia-3d", {{ItNumSetting(230,1)}}},
        {"Ib", {{ItNumSetting(9,15)}, {ItNumSetting(9,18)}}},
        {"Iba2", {{ItNumSetting(45,1)}, {ItNumSetting(45,2)}}},
        {"Ibam", {{ItNumSetting(72,1)}, {ItNumSetting(72,2)}}},
        {"Ibca", {{ItNumSetting(73,1)}, {ItNumSetting(73,3)}, {ItNumSetting(73,5)}}},
        {"Ibm2", {{ItNumSetting(46,2)}}},
        {"Ibmm", {{ItNumSetting(74,3)}}},
        {"Ic", {{ItNumSetting(9,14)}, {ItNumSetting(9,17)}}},
        {"Ic2a", {{ItNumSetting(45,5)}, {ItNumSetting(45,6)}}},
        {"Ic2m", {{ItNumSetting(46,5)}}},
        {"Icab", {{ItNumSetting(73,2)}, {ItNumSetting(73,4)}, {ItNumSetting(73,6)}}},
        {"Icma", {{ItNumSetting(72,5)}, {ItNumSetting(72,6)}}},
        {"Icmm", {{ItNumSetting(74,4)}}},
        {"Im", {{ItNumSetting(8,13)}, {ItNumSetting(8,14)}, {ItNumSetting(8,15)}, {ItNumSetting(8,16)}, {ItNumSetting(8,17)}, {ItNumSetting(8,18)}}},
        {"Im-3", {{ItNumSetting(204,1)}}},
        {"Im-3m", {{ItNumSetting(229,1)}}},
        {"Im2a", {{ItNumSetting(46,6)}}},
        {"Im2m", {{ItNumSetting(44,5)}, {ItNumSetting(44,6)}}},
        {"Ima2", {{ItNumSetting(46,1)}}},
        {"Imam", {{ItNumSetting(74,6)}}},
        {"Imcb", {{ItNumSetting(72,3)}, {ItNumSetting(72,4)}}},
        {"Imcm", {{ItNumSetting(74,5)}}},
        {"Imm2", {{ItNumSetting(44,1)}, {ItNumSetting(44,2)}}},
        {"Imma", {{ItNumSetting(74,1)}}},
        {"Immb", {{ItNumSetting(74,2)}}},
        {"Immm", {{ItNumSetting(71,1)}, {ItNumSetting(71,2)}, {ItNumSetting(71,3)}, {ItNumSetting(71,4)}, {ItNumSetting(71,5)}, {ItNumSetting(71,6)}}},
        {"P-1", {{ItNumSetting(2,1)}}},
        {"P-3", {{ItNumSetting(147,1)}}},
        {"P-31c", {{ItNumSetting(163,1)}}},
        {"P-31m", {{ItNumSetting(162,1)}}},
        {"P-3c1", {{ItNumSetting(165,1)}}},
        {"P-3m1", {{ItNumSetting(164,1)}}},
        {"P-4", {{ItNumSetting(81,1)}}},
        {"P-421c", {{ItNumSetting(114,1)}}},
        {"P-421m", {{ItNumSetting(113,1)}}},
        {"P-42c", {{ItNumSetting(112,1)}}},
        {"P-42m", {{ItNumSetting(111,1)}}},
        {"P-43m", {{ItNumSetting(215,1)}}},
        {"P-43n", {{ItNumSetting(218,1)}}},
        {"P-4b2", {{ItNumSetting(117,1)}}},
        {"P-4c2", {{ItNumSetting(116,1)}}},
        {"P-4m2", {{ItNumSetting(115,1)}}},
        {"P-4n2", {{ItNumSetting(118,1)}}},
        {"P-6", {{ItNumSetting(174,1)}}},
        {"P-62c", {{ItNumSetting(190,1)}}},
        {"P-62m", {{ItNumSetting(189,1)}}},
        {"P-6c2", {{ItNumSetting(188,1)}}},
        {"P-6m2", {{ItNumSetting(187,1)}}},
        {"P1", {{ItNumSetting(1,1)}}},
        {"P2", {{ItNumSetting(3,1)}, {ItNumSetting(3,2)}, {ItNumSetting(3,3)}, {ItNumSetting(3,4)}, {ItNumSetting(3,5)}, {ItNumSetting(3,6)}}},
        {"P2/a", {{ItNumSetting(13,2)}, {ItNumSetting(13,3)}, {ItNumSetting(13,13)}, {ItNumSetting(13,16)}}},
        {"P2/b", {{ItNumSetting(13,4)}, {ItNumSetting(13,5)}, {ItNumSetting(13,15)}, {ItNumSetting(13,18)}}},
        {"P2/c", {{ItNumSetting(13,1)}, {ItNumSetting(13,6)}, {ItNumSetting(13,14)}, {ItNumSetting(13,17)}}},
        {"P2/m", {{ItNumSetting(10,1)}, {ItNumSetting(10,2)}, {ItNumSetting(10,3)}, {ItNumSetting(10,4)}, {ItNumSetting(10,5)}, {ItNumSetting(10,6)}}},
        {"P2/n", {{ItNumSetting(13,7)}, {ItNumSetting(13,8)}, {ItNumSetting(13,9)}, {ItNumSetting(13,10)}, {ItNumSetting(13,11)}, {ItNumSetting(13,12)}}},
        {"P21", {{ItNumSetting(4,1)}, {ItNumSetting(4,2)}, {ItNumSetting(4,3)}, {ItNumSetting(4,4)}, {ItNumSetting(4,5)}, {ItNumSetting(4,6)}}},
        {"P21/a", {{ItNumSetting(14,2)}, {ItNumSetting(14,3)}, {ItNumSetting(14,13)}, {ItNumSetting(14,16)}}},
        {"P21/b", {{ItNumSetting(14,4)}, {ItNumSetting(14,5)}, {ItNumSetting(14,15)}, {ItNumSetting(14,18)}}},
        {"P21/c", {{ItNumSetting(14,1)}, {ItNumSetting(14,6)}, {ItNumSetting(14,14)}, {ItNumSetting(14,17)}}},
        {"P21/m", {{ItNumSetting(11,1)}, {ItNumSetting(11,2)}, {ItNumSetting(11,3)}, {ItNumSetting(11,4)}, {ItNumSetting(11,5)}, {ItNumSetting(11,6)}}},
        {"P21/n", {{ItNumSetting(14,7)}, {ItNumSetting(14,8)}, {ItNumSetting(14,9)}, {ItNumSetting(14,10)}, {ItNumSetting(14,11)}, {ItNumSetting(14,12)}}},
        {"P21212", {{ItNumSetting(18,1)}, {ItNumSetting(18,2)}}},
        {"P212121", {{ItNumSetting(19,1)}, {ItNumSetting(19,2)}, {ItNumSetting(19,3)}, {ItNumSetting(19,4)}, {ItNumSetting(19,5)}, {ItNumSetting(19,6)}}},
        {"P2122", {{ItNumSetting(17,3)}, {ItNumSetting(17,4)}}},
        {"P21221", {{ItNumSetting(18,5)}, {ItNumSetting(18,6)}}},
        {"P213", {{ItNumSetting(198,1)}}},
        {"P21ab", {{ItNumSetting(29,3)}}},
        {"P21am", {{ItNumSetting(26,4)}}},
        {"P21ca", {{ItNumSetting(29,4)}}},
        {"P21cn", {{ItNumSetting(33,4)}}},
        {"P21ma", {{ItNumSetting(26,3)}}},
        {"P21mn", {{ItNumSetting(31,3)}}},
        {"P21nb", {{ItNumSetting(33,3)}}},
        {"P21nm", {{ItNumSetting(31,4)}}},
        {"P2212", {{ItNumSetting(17,5)}, {ItNumSetting(17,6)}}},
        {"P22121", {{ItNumSetting(18,3)}, {ItNumSetting(18,4)}}},
        {"P222", {{ItNumSetting(16,1)}, {ItNumSetting(16,2)}, {ItNumSetting(16,3)}, {ItNumSetting(16,4)}, {ItNumSetting(16,5)}, {ItNumSetting(16,6)}}},
        {"P2221", {{ItNumSetting(17,1)}, {ItNumSetting(17,2)}}},
        {"P23", {{ItNumSetting(195,1)}}},
        {"P2aa", {{ItNumSetting(27,3)}, {ItNumSetting(27,4)}}},
        {"P2an", {{ItNumSetting(30,4)}}},
        {"P2cb", {{ItNumSetting(32,3)}, {ItNumSetting(32,4)}}},
        {"P2cm", {{ItNumSetting(28,4)}}},
        {"P2mb", {{ItNumSetting(28,3)}}},
        {"P2mm", {{ItNumSetting(25,3)}, {ItNumSetting(25,4)}}},
        {"P2na", {{ItNumSetting(30,3)}}},
        {"P2nn", {{ItNumSetting(34,3)}, {ItNumSetting(34,4)}}},
        {"P3", {{ItNumSetting(143,1)}}},
        {"P31", {{ItNumSetting(144,1)}}},
        {"P3112", {{ItNumSetting(151,1)}}},
        {"P312", {{ItNumSetting(149,1)}}},
        {"P3121", {{ItNumSetting(152,1)}}},
        {"P31c", {{ItNumSetting(159,1)}}},
        {"P31m", {{ItNumSetting(157,1)}}},
        {"P32", {{ItNumSetting(145,1)}}},
        {"P321", {{ItNumSetting(150,1)}}},
        {"P3212", {{ItNumSetting(153,1)}}},
        {"P3221", {{ItNumSetting(154,1)}}},
        {"P3c1", {{ItNumSetting(158,1)}}},
        {"P3m1", {{ItNumSetting(156,1)}}},
        {"P4", {{ItNumSetting(75,1)}}},
        {"P4/m", {{ItNumSetting(83,1)}}},
        {"P4/mbm", {{ItNumSetting(127,1)}}},
        {"P4/mcc", {{ItNumSetting(124,1)}}},
        {"P4/mmm", {{ItNumSetting(123,1)}}},
        {"P4/mnc", {{ItNumSetting(128,1)}}},
        {"P42/mnm", {{ItNumSetting(136,1)}}},
        {"P4/n", {{ItNumSetting(85,1)}, {ItNumSetting(85,2)}}},
        {"P4/nbm", {{ItNumSetting(125,1)}, {ItNumSetting(125,2)}}},
        {"P4/ncc", {{ItNumSetting(130,1)}, {ItNumSetting(130,2)}}},
        {"P4/nmm", {{ItNumSetting(129,1)}, {ItNumSetting(129,2)}}},
        {"P4/nnc", {{ItNumSetting(126,1)}, {ItNumSetting(126,2)}}},
        {"P41", {{ItNumSetting(76,1)}}},
        {"P41212", {{ItNumSetting(92,1)}}},
        {"P4122", {{ItNumSetting(91,1)}}},
        {"P4132", {{ItNumSetting(213,1)}}},
        {"P42", {{ItNumSetting(77,1)}}},
        {"P42/m", {{ItNumSetting(84,1)}}},
        {"P42/mbc", {{ItNumSetting(135,1)}}},
        {"P42/mcm", {{ItNumSetting(132,1)}}},
        {"P42/mmc", {{ItNumSetting(131,1)}}},
        {"P42/n", {{ItNumSetting(86,1)}, {ItNumSetting(86,2)}}},
        {"P42/nbc", {{ItNumSetting(133,1)}, {ItNumSetting(133,2)}}},
        {"P42/ncm", {{ItNumSetting(138,1)}, {ItNumSetting(138,2)}}},
        {"P42/nmc", {{ItNumSetting(137,1)}, {ItNumSetting(137,2)}}},
        {"P42/nnm", {{ItNumSetting(134,1)}, {ItNumSetting(134,2)}}},
        {"P4212", {{ItNumSetting(90,1)}}},
        {"P422", {{ItNumSetting(89,1)}}},
        {"P42212", {{ItNumSetting(94,1)}}},
        {"P4222", {{ItNumSetting(93,1)}}},
        {"P4232", {{ItNumSetting(208,1)}}},
        {"P42bc", {{ItNumSetting(106,1)}}},
        {"P42cm", {{ItNumSetting(101,1)}}},
        {"P42mc", {{ItNumSetting(105,1)}}},
        {"P42nm", {{ItNumSetting(102,1)}}},
        {"P43", {{ItNumSetting(78,1)}}},
        {"P432", {{ItNumSetting(207,1)}}},
        {"P43212", {{ItNumSetting(96,1)}}},
        {"P4322", {{ItNumSetting(95,1)}}},
        {"P4332", {{ItNumSetting(212,1)}}},
        {"P4bm", {{ItNumSetting(100,1)}}},
        {"P4cc", {{ItNumSetting(103,1)}}},
        {"P4mm", {{ItNumSetting(99,1)}}},
        {"P4nc", {{ItNumSetting(104,1)}}},
        {"P6", {{ItNumSetting(168,1)}}},
        {"P6/m", {{ItNumSetting(175,1)}}},
        {"P6/mcc", {{ItNumSetting(192,1)}}},
        {"P6/mmm", {{ItNumSetting(191,1)}}},
        {"P61", {{ItNumSetting(169,1)}}},
        {"P6122", {{ItNumSetting(178,1)}}},
        {"P62", {{ItNumSetting(171,1)}}},
        {"P622", {{ItNumSetting(177,1)}}},
        {"P6222", {{ItNumSetting(180,1)}}},
        {"P63", {{ItNumSetting(173,1)}}},
        {"P63/m", {{ItNumSetting(176,1)}}},
        {"P63/mcm", {{ItNumSetting(193,1)}}},
        {"P63/mmc", {{ItNumSetting(194,1)}}},
        {"P6322", {{ItNumSetting(182,1)}}},
        {"P63cm", {{ItNumSetting(185,1)}}},
        {"P63mc", {{ItNumSetting(186,1)}}},
        {"P64", {{ItNumSetting(172,1)}}},
        {"P6422", {{ItNumSetting(181,1)}}},
        {"P65", {{ItNumSetting(170,1)}}},
        {"P6522", {{ItNumSetting(179,1)}}},
        {"P6cc", {{ItNumSetting(184,1)}}},
        {"P6mm", {{ItNumSetting(183,1)}}},
        {"Pa", {{ItNumSetting(7,2)}, {ItNumSetting(7,3)}, {ItNumSetting(7,13)}, {ItNumSetting(7,16)}}},
        {"Pa-3", {{ItNumSetting(205,1)}}},
        {"Pb", {{ItNumSetting(7,4)}, {ItNumSetting(7,5)}, {ItNumSetting(7,15)}, {ItNumSetting(7,18)}}},
        {"Pb21a", {{ItNumSetting(29,6)}}},
        {"Pb21m", {{ItNumSetting(26,5)}}},
        {"Pb2b", {{ItNumSetting(27,5)}, {ItNumSetting(27,6)}}},
        {"Pb2n", {{ItNumSetting(30,5)}}},
        {"Pba2", {{ItNumSetting(32,1)}, {ItNumSetting(32,2)}}},
        {"Pbaa", {{ItNumSetting(54,3)}}},
        {"Pbab", {{ItNumSetting(54,6)}}},
        {"Pbam", {{ItNumSetting(55,1)}, {ItNumSetting(55,2)}}},
        {"Pban", {{ItNumSetting(50,1)}, {ItNumSetting(50,2)}, {ItNumSetting(50,3)}, {ItNumSetting(50,4)}}},
        {"Pbc21", {{ItNumSetting(29,2)}}},
        {"Pbca", {{ItNumSetting(61,1)}, {ItNumSetting(61,3)}, {ItNumSetting(61,5)}}},
        {"Pbcb", {{ItNumSetting(54,5)}}},
        {"Pbcm", {{ItNumSetting(57,1)}}},
        {"Pbcn", {{ItNumSetting(60,1)}}},
        {"Pbm2", {{ItNumSetting(28,2)}}},
        {"Pbma", {{ItNumSetting(57,5)}}},
        {"Pbmb", {{ItNumSetting(49,5)}, {ItNumSetting(49,6)}}},
        {"Pbmm", {{ItNumSetting(51,3)}}},
        {"Pbmn", {{ItNumSetting(53,3)}}},
        {"Pbn21", {{ItNumSetting(33,2)}}},
        {"Pbna", {{ItNumSetting(60,5)}}},
        {"Pbnb", {{ItNumSetting(56,5)}, {ItNumSetting(56,6)}}},
        {"Pbnm", {{ItNumSetting(62,3)}}},
        {"Pbnn", {{ItNumSetting(52,3)}}},
        {"Pc", {{ItNumSetting(7,1)}, {ItNumSetting(7,6)}, {ItNumSetting(7,14)}, {ItNumSetting(7,17)}}},
        {"Pc21b", {{ItNumSetting(29,5)}}},
        {"Pc21n", {{ItNumSetting(33,5)}}},
        {"Pc2a", {{ItNumSetting(32,5)}, {ItNumSetting(32,6)}}},
        {"Pc2m", {{ItNumSetting(28,5)}}},
        {"Pca21", {{ItNumSetting(29,1)}}},
        {"Pcaa", {{ItNumSetting(54,4)}}},
        {"Pcab", {{ItNumSetting(61,2)}, {ItNumSetting(61,4)}, {ItNumSetting(61,6)}}},
        {"Pcam", {{ItNumSetting(57,2)}}},
        {"Pcan", {{ItNumSetting(60,2)}}},
        {"Pcc2", {{ItNumSetting(27,1)}, {ItNumSetting(27,2)}}},
        {"Pcca", {{ItNumSetting(54,1)}}},
        {"Pccb", {{ItNumSetting(54,2)}}},
        {"Pccm", {{ItNumSetting(49,1)}, {ItNumSetting(49,2)}}},
        {"Pccn", {{ItNumSetting(56,1)}, {ItNumSetting(56,2)}}},
        {"Pcm21", {{ItNumSetting(26,2)}}},
        {"Pcma", {{ItNumSetting(55,5)}, {ItNumSetting(55,6)}}},
        {"Pcmb", {{ItNumSetting(57,6)}}},
        {"Pcmm", {{ItNumSetting(51,4)}}},
        {"Pcmn", {{ItNumSetting(62,4)}}},
        {"Pcn2", {{ItNumSetting(30,2)}}},
        {"Pcna", {{ItNumSetting(50,9)}, {ItNumSetting(50,10)}, {ItNumSetting(50,11)}, {ItNumSetting(50,12)}}},
        {"Pcnb", {{ItNumSetting(60,6)}}},
        {"Pcnm", {{ItNumSetting(53,4)}}},
        {"Pcnn", {{ItNumSetting(52,4)}}},
        {"Pm", {{ItNumSetting(6,1)}, {ItNumSetting(6,2)}, {ItNumSetting(6,3)}, {ItNumSetting(6,4)}, {ItNumSetting(6,5)}, {ItNumSetting(6,6)}}},
        {"Pm-3", {{ItNumSetting(200,1)}}},
        {"Pm-3m", {{ItNumSetting(221,1)}}},
        {"Pm-3n", {{ItNumSetting(223,1)}}},
        {"Pm21b", {{ItNumSetting(26,6)}}},
        {"Pm21n", {{ItNumSetting(31,6)}}},
        {"Pm2a", {{ItNumSetting(28,6)}}},
        {"Pm2m", {{ItNumSetting(25,5)}, {ItNumSetting(25,6)}}},
        {"Pma2", {{ItNumSetting(28,1)}}},
        {"Pmaa", {{ItNumSetting(49,3)}, {ItNumSetting(49,4)}}},
        {"Pmab", {{ItNumSetting(57,4)}}},
        {"Pmam", {{ItNumSetting(51,6)}}},
        {"Pman", {{ItNumSetting(53,6)}}},
        {"Pmc21", {{ItNumSetting(26,1)}}},
        {"Pmca", {{ItNumSetting(57,3)}}},
        {"Pmcb", {{ItNumSetting(55,3)}, {ItNumSetting(55,4)}}},
        {"Pmcm", {{ItNumSetting(51,5)}}},
        {"Pmcn", {{ItNumSetting(62,5)}}},
        {"Pmm2", {{ItNumSetting(25,1)}, {ItNumSetting(25,2)}}},
        {"Pmma", {{ItNumSetting(51,1)}}},
        {"Pmmb", {{ItNumSetting(51,2)}}},
        {"Pmmm", {{ItNumSetting(47,1)}, {ItNumSetting(47,2)}, {ItNumSetting(47,3)}, {ItNumSetting(47,4)}, {ItNumSetting(47,5)}, {ItNumSetting(47,6)}}},
        {"Pmmn", {{ItNumSetting(59,1)}, {ItNumSetting(59,2)}, {ItNumSetting(59,3)}, {ItNumSetting(59,4)}}},
        {"Pmn21", {{ItNumSetting(31,1)}}},
        {"Pmna", {{ItNumSetting(53,1)}}},
        {"Pmnb", {{ItNumSetting(62,2)}}},
        {"Pmnm", {{ItNumSetting(59,9)}, {ItNumSetting(59,10)}, {ItNumSetting(59,11)}, {ItNumSetting(59,12)}}},
        {"Pmnn", {{ItNumSetting(58,3)}, {ItNumSetting(58,4)}}},
        {"Pn", {{ItNumSetting(7,7)}, {ItNumSetting(7,8)}, {ItNumSetting(7,9)}, {ItNumSetting(7,10)}, {ItNumSetting(7,11)}, {ItNumSetting(7,12)}}},
        {"Pn-3", {{ItNumSetting(201,1)}, {ItNumSetting(201,2)}}},
        {"Pn-3m", {{ItNumSetting(224,1)}, {ItNumSetting(224,2)}}},
        {"Pn-3n", {{ItNumSetting(222,1)}, {ItNumSetting(222,2)}}},
        {"Pn21a", {{ItNumSetting(33,6)}}},
        {"Pn21m", {{ItNumSetting(31,5)}}},
        {"Pn2b", {{ItNumSetting(30,6)}}},
        {"Pn2n", {{ItNumSetting(34,5)}, {ItNumSetting(34,6)}}},
        {"Pna21", {{ItNumSetting(33,1)}}},
        {"Pnaa", {{ItNumSetting(56,3)}, {ItNumSetting(56,4)}}},
        {"Pnab", {{ItNumSetting(60,4)}}},
        {"Pnam", {{ItNumSetting(62,6)}}},
        {"Pnan", {{ItNumSetting(52,6)}}},
        {"Pnc2", {{ItNumSetting(30,1)}}},
        {"Pnca", {{ItNumSetting(60,3)}}},
        {"Pncb", {{ItNumSetting(50,5)}, {ItNumSetting(50,6)}, {ItNumSetting(50,7)}, {ItNumSetting(50,8)}}},
        {"Pncm", {{ItNumSetting(53,5)}}},
        {"Pncn", {{ItNumSetting(52,5)}}},
        {"Pnm21", {{ItNumSetting(31,2)}}},
        {"Pnma", {{ItNumSetting(62,1)}}},
        {"Pnmb", {{ItNumSetting(53,2)}}},
        {"Pnmm", {{ItNumSetting(59,5)}, {ItNumSetting(59,6)}, {ItNumSetting(59,7)}, {ItNumSetting(59,8)}}},
        {"Pnmn", {{ItNumSetting(58,5)}, {ItNumSetting(58,6)}}},
        {"Pnn2", {{ItNumSetting(34,1)}, {ItNumSetting(34,2)}}},
        {"Pnna", {{ItNumSetting(52,1)}}},
        {"Pnnb", {{ItNumSetting(52,2)}}},
        {"Pnnm", {{ItNumSetting(58,1)}, {ItNumSetting(58,2)}}},
        {"Pnnn", {{ItNumSetting(48,1)}, {ItNumSetting(48,2)}, {ItNumSetting(48,3)}, {ItNumSetting(48,4)}, {ItNumSetting(48,5)}, {ItNumSetting(48,6)}, {ItNumSetting(48,7)}, {ItNumSetting(48,8)}, {ItNumSetting(48,9)}, {ItNumSetting(48,10)}, {ItNumSetting(48,11)}, {ItNumSetting(48,12)}}},
        {"R-3", {{ItNumSetting(148,1)}, {ItNumSetting(148,2)}}},
        {"R-3c", {{ItNumSetting(167,1)}, {ItNumSetting(167,2)}}},
        {"R-3m", {{ItNumSetting(166,1)}, {ItNumSetting(166,2)}}},
        {"R3", {{ItNumSetting(146,1)}, {ItNumSetting(146,2)}}},
        {"R32", {{ItNumSetting(155,1)}, {ItNumSetting(155,2)}}},
        {"R3c", {{ItNumSetting(161,1)}, {ItNumSetting(161,2)}}},
        {"R3m", {{ItNumSetting(160,1)}, {ItNumSetting(160,2)}}}
    };

    // full HM symbols in CIF format (screw axes as 21)
    const QMap<QString, QList<ItNumSetting> > hmFull {
        {"A-1", {{ItNumSetting(2,3)}}},
        {"A1", {{ItNumSetting(1,3)}}},
        {"A112", {{ItNumSetting(5,3)}, {ItNumSetting(5,10)}}},
        {"A112/a", {{ItNumSetting(15,3)}}},
        {"A112/m", {{ItNumSetting(12,3)}, {ItNumSetting(12,10)}}},
        {"A112/n", {{ItNumSetting(15,10)}}},
        {"A11a", {{ItNumSetting(9,3)}}},
        {"A11m", {{ItNumSetting(8,3)}, {ItNumSetting(8,10)}}},
        {"A11n", {{ItNumSetting(9,10)}}},
        {"A12/a1", {{ItNumSetting(15,2)}}},
        {"A12/m1", {{ItNumSetting(12,2)}, {ItNumSetting(12,7)}}},
        {"A12/n1", {{ItNumSetting(15,7)}}},
        {"A121", {{ItNumSetting(5,2)}, {ItNumSetting(5,7)}}},
        {"A1a1", {{ItNumSetting(9,2)}}},
        {"A1m1", {{ItNumSetting(8,2)}, {ItNumSetting(8,7)}}},
        {"A1n1", {{ItNumSetting(9,7)}}},
        {"A2/b2/a2/a", {{ItNumSetting(68,5)}, {ItNumSetting(68,6)}}},
        {"A2/b2/m2/m", {{ItNumSetting(67,3)}}},
        {"A2/c2/a2/a", {{ItNumSetting(68,7)}, {ItNumSetting(68,8)}}},
        {"A2/c2/m2/m", {{ItNumSetting(67,4)}}},
        {"A2/m2/a2/a", {{ItNumSetting(66,3)}, {ItNumSetting(66,4)}}},
        {"A2/m2/m2/m", {{ItNumSetting(65,3)}, {ItNumSetting(65,4)}}},
        {"A222", {{ItNumSetting(21,3)}, {ItNumSetting(21,4)}}},
        {"A21/b2/m2/a", {{ItNumSetting(64,3)}}},
        {"A21/c2/a2/m", {{ItNumSetting(64,4)}}},
        {"A21/m2/a2/m", {{ItNumSetting(63,4)}}},
        {"A21/m2/m2/a", {{ItNumSetting(63,3)}}},
        {"A2122", {{ItNumSetting(20,3)}, {ItNumSetting(20,4)}}},
        {"A21am", {{ItNumSetting(36,4)}}},
        {"A21ma", {{ItNumSetting(36,3)}}},
        {"A2aa", {{ItNumSetting(37,3)}, {ItNumSetting(37,4)}}},
        {"A2mm", {{ItNumSetting(35,3)}, {ItNumSetting(35,4)}}},
        {"Aba2", {{ItNumSetting(41,1)}}},
        {"Abm2", {{ItNumSetting(39,1)}}},
        {"Ac2a", {{ItNumSetting(41,6)}}},
        {"Ac2m", {{ItNumSetting(39,6)}}},
        {"Am2a", {{ItNumSetting(40,6)}}},
        {"Am2m", {{ItNumSetting(38,6)}}},
        {"Ama2", {{ItNumSetting(40,1)}}},
        {"Amm2", {{ItNumSetting(38,1)}}},
        {"B-1", {{ItNumSetting(2,4)}}},
        {"B1", {{ItNumSetting(1,4)}}},
        {"B112", {{ItNumSetting(5,4)}, {ItNumSetting(5,9)}}},
        {"B112/b", {{ItNumSetting(15,4)}}},
        {"B112/m", {{ItNumSetting(12,4)}, {ItNumSetting(12,9)}}},
        {"B112/n", {{ItNumSetting(15,9)}}},
        {"B11b", {{ItNumSetting(9,4)}}},
        {"B11m", {{ItNumSetting(8,4)}, {ItNumSetting(8,9)}}},
        {"B11n", {{ItNumSetting(9,9)}}},
        {"B2/b11", {{ItNumSetting(15,5)}}},
        {"B2/b2/a2/b", {{ItNumSetting(68,11)}, {ItNumSetting(68,12)}}},
        {"B2/b2/c2/b", {{ItNumSetting(68,9)}, {ItNumSetting(68,10)}}},
        {"B2/b2/m2/b", {{ItNumSetting(66,5)}, {ItNumSetting(66,6)}}},
        {"B2/b21/c2/m", {{ItNumSetting(64,5)}}},
        {"B2/b21/m2/m", {{ItNumSetting(63,5)}}},
        {"B2/m11", {{ItNumSetting(12,5)}, {ItNumSetting(12,12)}}},
        {"B2/m2/a2/m", {{ItNumSetting(67,6)}}},
        {"B2/m2/c2/m", {{ItNumSetting(67,5)}}},
        {"B2/m2/m2/m", {{ItNumSetting(65,5)}, {ItNumSetting(65,6)}}},
        {"B2/m21/a2/b", {{ItNumSetting(64,6)}}},
        {"B2/m21/m2/b", {{ItNumSetting(63,6)}}},
        {"B2/n11", {{ItNumSetting(15,12)}}},
        {"B211", {{ItNumSetting(5,5)}, {ItNumSetting(5,12)}}},
        {"B222", {{ItNumSetting(21,5)}, {ItNumSetting(21,6)}}},
        {"B2212", {{ItNumSetting(20,5)}, {ItNumSetting(20,6)}}},
        {"B2cb", {{ItNumSetting(41,3)}}},
        {"B2cm", {{ItNumSetting(39,3)}}},
        {"B2mb", {{ItNumSetting(40,3)}}},
        {"B2mm", {{ItNumSetting(38,3)}}},
        {"Bb11", {{ItNumSetting(9,5)}}},
        {"Bb21m", {{ItNumSetting(36,5)}}},
        {"Bb2b", {{ItNumSetting(37,5)}, {ItNumSetting(37,6)}}},
        {"Bba2", {{ItNumSetting(41,2)}}},
        {"Bbm2", {{ItNumSetting(40,2)}}},
        {"Bm11", {{ItNumSetting(8,5)}, {ItNumSetting(8,12)}}},
        {"Bm21b", {{ItNumSetting(36,6)}}},
        {"Bm2m", {{ItNumSetting(35,5)}, {ItNumSetting(35,6)}}},
        {"Bma2", {{ItNumSetting(39,2)}}},
        {"Bmm2", {{ItNumSetting(38,2)}}},
        {"Bn11", {{ItNumSetting(9,12)}}},
        {"C-1", {{ItNumSetting(2,2)}}},
        {"C1", {{ItNumSetting(1,2)}}},
        {"C12/c1", {{ItNumSetting(15,1)}}},
        {"C12/m1", {{ItNumSetting(12,1)}, {ItNumSetting(12,8)}}},
        {"C12/n1", {{ItNumSetting(15,8)}}},
        {"C121", {{ItNumSetting(5,1)}, {ItNumSetting(5,8)}}},
        {"C1c1", {{ItNumSetting(9,1)}}},
        {"C1m1", {{ItNumSetting(8,1)}, {ItNumSetting(8,8)}}},
        {"C1n1", {{ItNumSetting(9,8)}}},
        {"C2/c11", {{ItNumSetting(15,6)}}},
        {"C2/c2/c2/a", {{ItNumSetting(68,1)}, {ItNumSetting(68,2)}}},
        {"C2/c2/c2/b", {{ItNumSetting(68,3)}, {ItNumSetting(68,4)}}},
        {"C2/c2/c2/m", {{ItNumSetting(66,1)}, {ItNumSetting(66,2)}}},
        {"C2/c2/m21/b", {{ItNumSetting(64,2)}}},
        {"C2/c2/m21m", {{ItNumSetting(63,2)}}},
        {"C2/m11", {{ItNumSetting(12,6)}, {ItNumSetting(12,11)}}},
        {"C2/m2/c21/a", {{ItNumSetting(64,1)}}},
        {"C2/m2/c21/m", {{ItNumSetting(63,1)}}},
        {"C2/m2/m2/a", {{ItNumSetting(67,1)}}},
        {"C2/m2/m2/b", {{ItNumSetting(67,2)}}},
        {"C2/m2/m2/m", {{ItNumSetting(65,1)}, {ItNumSetting(65,2)}}},
        {"C2/n11", {{ItNumSetting(15,11)}}},
        {"C211", {{ItNumSetting(5,6)}, {ItNumSetting(5,11)}}},
        {"C222", {{ItNumSetting(21,1)}, {ItNumSetting(21,2)}}},
        {"C2221", {{ItNumSetting(20,1)}, {ItNumSetting(20,2)}}},
        {"C2cb", {{ItNumSetting(41,4)}}},
        {"C2cm", {{ItNumSetting(40,4)}}},
        {"C2mb", {{ItNumSetting(39,4)}}},
        {"C2mm", {{ItNumSetting(38,4)}}},
        {"Cc11", {{ItNumSetting(9,6)}}},
        {"Cc2a", {{ItNumSetting(41,5)}}},
        {"Cc2m", {{ItNumSetting(40,5)}}},
        {"Ccc2", {{ItNumSetting(37,1)}, {ItNumSetting(37,2)}}},
        {"Ccm21", {{ItNumSetting(36,2)}}},
        {"Cm11", {{ItNumSetting(8,6)}, {ItNumSetting(8,11)}}},
        {"Cm2a", {{ItNumSetting(39,5)}}},
        {"Cm2m", {{ItNumSetting(38,5)}}},
        {"Cmc21", {{ItNumSetting(36,1)}}},
        {"Cmm2", {{ItNumSetting(35,1)}, {ItNumSetting(35,2)}}},
        {"Cn11", {{ItNumSetting(9,11)}}},
        {"F-1", {{ItNumSetting(2,6)}}},
        {"F-43c", {{ItNumSetting(219,1)}}},
        {"F-43m", {{ItNumSetting(216,1)}}},
        {"F1", {{ItNumSetting(1,6)}}},
        {"F121", {{ItNumSetting(3,7)}}},
        {"F2/d-3", {{ItNumSetting(203,1)}, {ItNumSetting(203,2)}}},
        {"F2/d2/d2/d", {{ItNumSetting(70,1)}, {ItNumSetting(70,2)}, {ItNumSetting(70,3)}, {ItNumSetting(70,4)}, {ItNumSetting(70,5)}, {ItNumSetting(70,6)}, {ItNumSetting(70,7)}, {ItNumSetting(70,8)}, {ItNumSetting(70,9)}, {ItNumSetting(70,10)}, {ItNumSetting(70,11)}, {ItNumSetting(70,12)}}},
        {"F2/m-3", {{ItNumSetting(202,1)}}},
        {"F2/m2/m2/m", {{ItNumSetting(69,1)}, {ItNumSetting(69,2)}, {ItNumSetting(69,3)}, {ItNumSetting(69,4)}, {ItNumSetting(69,5)}, {ItNumSetting(69,6)}}},
        {"F222", {{ItNumSetting(22,1)}, {ItNumSetting(22,2)}, {ItNumSetting(22,3)}, {ItNumSetting(22,4)}, {ItNumSetting(22,5)}, {ItNumSetting(22,6)}}},
        {"F23", {{ItNumSetting(196,1)}}},
        {"F2dd", {{ItNumSetting(43,3)}, {ItNumSetting(43,4)}}},
        {"F2mm", {{ItNumSetting(42,3)}, {ItNumSetting(42,4)}}},
        {"F4/m-32/c", {{ItNumSetting(226,1)}}},
        {"F4/m-32/m", {{ItNumSetting(225,1)}}},
        {"F432", {{ItNumSetting(209,1)}}},
        {"F41/d-32/c", {{ItNumSetting(228,1)}, {ItNumSetting(228,2)}}},
        {"F41/d-32/m", {{ItNumSetting(227,1)}, {ItNumSetting(227,2)}}},
        {"F4132", {{ItNumSetting(210,1)}}},
        {"Fd2d", {{ItNumSetting(43,5)}, {ItNumSetting(43,6)}}},
        {"Fdd2", {{ItNumSetting(43,1)}, {ItNumSetting(43,2)}}},
        {"Fm2m", {{ItNumSetting(42,5)}, {ItNumSetting(42,6)}}},
        {"Fmm2", {{ItNumSetting(42,1)}, {ItNumSetting(42,2)}}},
        {"I-1", {{ItNumSetting(2,5)}}},
        {"I-4", {{ItNumSetting(82,1)}}},
        {"I-42d", {{ItNumSetting(122,1)}}},
        {"I-42m", {{ItNumSetting(121,1)}}},
        {"I-43d", {{ItNumSetting(220,1)}}},
        {"I-43m", {{ItNumSetting(217,1)}}},
        {"I-4c2", {{ItNumSetting(120,1)}}},
        {"I-4m2", {{ItNumSetting(119,1)}}},
        {"I1", {{ItNumSetting(1,5)}}},
        {"I112", {{ItNumSetting(5,15)}, {ItNumSetting(5,16)}}},
        {"I112/a", {{ItNumSetting(15,16)}}},
        {"I112/b", {{ItNumSetting(15,15)}}},
        {"I112/m", {{ItNumSetting(12,15)}, {ItNumSetting(12,16)}}},
        {"I11a", {{ItNumSetting(9,16)}}},
        {"I11b", {{ItNumSetting(9,15)}}},
        {"I11m", {{ItNumSetting(8,15)}, {ItNumSetting(8,16)}}},
        {"I12/a1", {{ItNumSetting(15,13)}}},
        {"I12/c1", {{ItNumSetting(15,14)}}},
        {"I12/m1", {{ItNumSetting(12,13)}, {ItNumSetting(12,14)}}},
        {"I121", {{ItNumSetting(5,13)}, {ItNumSetting(5,14)}}},
        {"I1a1", {{ItNumSetting(9,13)}}},
        {"I1c1", {{ItNumSetting(9,14)}}},
        {"I1m1", {{ItNumSetting(8,13)}, {ItNumSetting(8,14)}}},
        {"I2/b11", {{ItNumSetting(15,18)}}},
        {"I2/b2/a2/m", {{ItNumSetting(72,1)}, {ItNumSetting(72,2)}}},
        {"I2/c11", {{ItNumSetting(15,17)}}},
        {"I2/c2/m2/a", {{ItNumSetting(72,5)}, {ItNumSetting(72,6)}}},
        {"I2/m-3", {{ItNumSetting(204,1)}}},
        {"I2/m11", {{ItNumSetting(12,17)}, {ItNumSetting(12,18)}}},
        {"I2/m2/c2/b", {{ItNumSetting(72,3)}, {ItNumSetting(72,4)}}},
        {"I2/m2/m2/m", {{ItNumSetting(71,1)}, {ItNumSetting(71,2)}, {ItNumSetting(71,3)}, {ItNumSetting(71,4)}, {ItNumSetting(71,5)}, {ItNumSetting(71,6)}}},
        {"I211", {{ItNumSetting(5,17)}, {ItNumSetting(5,18)}}},
        {"I222", {{ItNumSetting(23,1)}, {ItNumSetting(23,2)}, {ItNumSetting(23,3)}, {ItNumSetting(23,4)}, {ItNumSetting(23,5)}, {ItNumSetting(23,6)}}},
        {"I23", {{ItNumSetting(197,1)}}},
        {"I21/a-3", {{ItNumSetting(206,1)}}},
        {"I21/b21/c21/a", {{ItNumSetting(73,1)}, {ItNumSetting(73,3)}, {ItNumSetting(73,5)}}},
        {"I21/b21/m21/m", {{ItNumSetting(74,3)}}},
        {"I21/c21/a21/b", {{ItNumSetting(73,2)}, {ItNumSetting(73,4)}, {ItNumSetting(73,6)}}},
        {"I21/c21/m21/m", {{ItNumSetting(74,4)}}},
        {"I21/m21/a21/m", {{ItNumSetting(74,6)}}},
        {"I21/m21/c21/m", {{ItNumSetting(74,5)}}},
        {"I21/m21/m21/a", {{ItNumSetting(74,1)}}},
        {"I21/m21/m21/b", {{ItNumSetting(74,2)}}},
        {"I212121", {{ItNumSetting(24,1)}, {ItNumSetting(24,2)}, {ItNumSetting(24,3)}, {ItNumSetting(24,4)}, {ItNumSetting(24,5)}, {ItNumSetting(24,6)}}},
        {"I213", {{ItNumSetting(199,1)}}},
        {"I2cb", {{ItNumSetting(45,3)}, {ItNumSetting(45,4)}}},
        {"I2cm", {{ItNumSetting(46,4)}}},
        {"I2mb", {{ItNumSetting(46,3)}}},
        {"I2mm", {{ItNumSetting(44,3)}, {ItNumSetting(44,4)}}},
        {"I4", {{ItNumSetting(79,1)}}},
        {"I4/m", {{ItNumSetting(87,1)}}},
        {"I4/m-32/m", {{ItNumSetting(229,1)}}},
        {"I4/m2/c2/m", {{ItNumSetting(140,1)}}},
        {"I4/m2/m2/m", {{ItNumSetting(139,1)}}},
        {"I422", {{ItNumSetting(97,1)}}},
        {"I432", {{ItNumSetting(211,1)}}},
        {"I41", {{ItNumSetting(80,1)}}},
        {"I41/a", {{ItNumSetting(88,1)}, {ItNumSetting(88,2)}}},
        {"I41/a-32/d", {{ItNumSetting(230,1)}}},
        {"I41/a2/c2/d", {{ItNumSetting(142,1)}, {ItNumSetting(142,2)}}},
        {"I41/a2/m2/d", {{ItNumSetting(141,1)}, {ItNumSetting(141,2)}}},
        {"I4122", {{ItNumSetting(98,1)}}},
        {"I4132", {{ItNumSetting(214,1)}}},
        {"I41cd", {{ItNumSetting(110,1)}}},
        {"I41md", {{ItNumSetting(109,1)}}},
        {"I4cm", {{ItNumSetting(108,1)}}},
        {"I4mm", {{ItNumSetting(107,1)}}},
        {"Ib11", {{ItNumSetting(9,18)}}},
        {"Iba2", {{ItNumSetting(45,1)}, {ItNumSetting(45,2)}}},
        {"Ibm2", {{ItNumSetting(46,2)}}},
        {"Ic11", {{ItNumSetting(9,17)}}},
        {"Ic2a", {{ItNumSetting(45,5)}, {ItNumSetting(45,6)}}},
        {"Ic2m", {{ItNumSetting(46,5)}}},
        {"Im11", {{ItNumSetting(8,17)}, {ItNumSetting(8,18)}}},
        {"Im2a", {{ItNumSetting(46,6)}}},
        {"Im2m", {{ItNumSetting(44,5)}, {ItNumSetting(44,6)}}},
        {"Ima2", {{ItNumSetting(46,1)}}},
        {"Imm2", {{ItNumSetting(44,1)}, {ItNumSetting(44,2)}}},
        {"P-1", {{ItNumSetting(2,1)}}},
        {"P-3", {{ItNumSetting(147,1)}}},
        {"P-312/c", {{ItNumSetting(163,1)}}},
        {"P-312/m", {{ItNumSetting(162,1)}}},
        {"P-32/c1", {{ItNumSetting(165,1)}}},
        {"P-32/m1", {{ItNumSetting(164,1)}}},
        {"P-4", {{ItNumSetting(81,1)}}},
        {"P-421c", {{ItNumSetting(114,1)}}},
        {"P-421m", {{ItNumSetting(113,1)}}},
        {"P-42c", {{ItNumSetting(112,1)}}},
        {"P-42m", {{ItNumSetting(111,1)}}},
        {"P-43m", {{ItNumSetting(215,1)}}},
        {"P-43n", {{ItNumSetting(218,1)}}},
        {"P-4b2", {{ItNumSetting(117,1)}}},
        {"P-4c2", {{ItNumSetting(116,1)}}},
        {"P-4m2", {{ItNumSetting(115,1)}}},
        {"P-4n2", {{ItNumSetting(118,1)}}},
        {"P-6", {{ItNumSetting(174,1)}}},
        {"P-62c", {{ItNumSetting(190,1)}}},
        {"P-62m", {{ItNumSetting(189,1)}}},
        {"P-6c2", {{ItNumSetting(188,1)}}},
        {"P-6m2", {{ItNumSetting(187,1)}}},
        {"P1", {{ItNumSetting(1,1)}}},
        {"P112", {{ItNumSetting(3,3)}, {ItNumSetting(3,4)}}},
        {"P112/a", {{ItNumSetting(13,3)}, {ItNumSetting(13,16)}}},
        {"P112/b", {{ItNumSetting(13,4)}, {ItNumSetting(13,15)}}},
        {"P112/m", {{ItNumSetting(10,3)}, {ItNumSetting(10,4)}}},
        {"P112/n", {{ItNumSetting(13,9)}, {ItNumSetting(13,10)}}},
        {"P1121", {{ItNumSetting(4,3)}, {ItNumSetting(4,4)}}},
        {"P1121/a", {{ItNumSetting(14,3)}, {ItNumSetting(14,16)}}},
        {"P1121/b", {{ItNumSetting(14,4)}, {ItNumSetting(14,15)}}},
        {"P1121/m", {{ItNumSetting(11,3)}, {ItNumSetting(11,4)}}},
        {"P1121/n", {{ItNumSetting(14,9)}, {ItNumSetting(14,10)}}},
        {"P11a", {{ItNumSetting(7,3)}, {ItNumSetting(7,16)}}},
        {"P11b", {{ItNumSetting(7,4)}, {ItNumSetting(7,15)}}},
        {"P11m", {{ItNumSetting(6,3)}, {ItNumSetting(6,4)}}},
        {"P11n", {{ItNumSetting(7,9)}, {ItNumSetting(7,10)}}},
        {"P12/a1", {{ItNumSetting(13,2)}, {ItNumSetting(13,13)}}},
        {"P12/c1", {{ItNumSetting(13,1)}, {ItNumSetting(13,14)}}},
        {"P12/m1", {{ItNumSetting(10,1)}, {ItNumSetting(10,2)}}},
        {"P12/n1", {{ItNumSetting(13,7)}, {ItNumSetting(13,8)}}},
        {"P121", {{ItNumSetting(3,1)}, {ItNumSetting(3,2)}}},
        {"P121/a1", {{ItNumSetting(14,2)}, {ItNumSetting(14,13)}}},
        {"P121/c1", {{ItNumSetting(14,1)}, {ItNumSetting(14,14)}}},
        {"P121/m1", {{ItNumSetting(11,1)}, {ItNumSetting(11,2)}}},
        {"P121/n1", {{ItNumSetting(14,7)}, {ItNumSetting(14,8)}}},
        {"P1211", {{ItNumSetting(4,1)}, {ItNumSetting(4,2)}}},
        {"P1a1", {{ItNumSetting(7,2)}, {ItNumSetting(7,13)}}},
        {"P1c1", {{ItNumSetting(7,1)}, {ItNumSetting(7,14)}}},
        {"P1m1", {{ItNumSetting(6,1)}, {ItNumSetting(6,2)}}},
        {"P1n1", {{ItNumSetting(7,7)}, {ItNumSetting(7,8)}}},
        {"P2/b11", {{ItNumSetting(13,5)}, {ItNumSetting(13,18)}}},
        {"P2/b2/a2/n", {{ItNumSetting(50,1)}, {ItNumSetting(50,2)}, {ItNumSetting(50,3)}, {ItNumSetting(50,4)}}},
        {"P2/b2/a21/b", {{ItNumSetting(54,6)}}},
        {"P2/b2/c21/b", {{ItNumSetting(54,5)}}},
        {"P2/b2/m2/b", {{ItNumSetting(49,5)}, {ItNumSetting(49,6)}}},
        {"P2/b2/n21/n", {{ItNumSetting(52,3)}}},
        {"P2/b21/a2/a", {{ItNumSetting(54,3)}}},
        {"P2/b21/c21/m", {{ItNumSetting(57,1)}}},
        {"P2/b21/m2/m", {{ItNumSetting(51,3)}}},
        {"P2/b21/n21/a", {{ItNumSetting(60,5)}}},
        {"P2/c11", {{ItNumSetting(13,6)}, {ItNumSetting(13,17)}}},
        {"P2/c2/c2/m", {{ItNumSetting(49,1)}, {ItNumSetting(49,2)}}},
        {"P2/c2/n2/a", {{ItNumSetting(50,9)}, {ItNumSetting(50,10)}, {ItNumSetting(50,11)}, {ItNumSetting(50,12)}}},
        {"P2/c2/n21/n", {{ItNumSetting(52,4)}}},
        {"P2/c21/a2/a", {{ItNumSetting(54,4)}}},
        {"P2/c21/a21/m", {{ItNumSetting(57,2)}}},
        {"P2/c21/m2/m", {{ItNumSetting(51,4)}}},
        {"P2/c21/n21/b", {{ItNumSetting(60,6)}}},
        {"P2/m-3", {{ItNumSetting(200,1)}}},
        {"P2/m11", {{ItNumSetting(10,5)}, {ItNumSetting(10,6)}}},
        {"P2/m2/a2/a", {{ItNumSetting(49,3)}, {ItNumSetting(49,4)}}},
        {"P2/m2/a21/m", {{ItNumSetting(51,6)}}},
        {"P2/m2/c21/m", {{ItNumSetting(51,5)}}},
        {"P2/m2/m2/m", {{ItNumSetting(47,1)}, {ItNumSetting(47,2)}, {ItNumSetting(47,3)}, {ItNumSetting(47,4)}, {ItNumSetting(47,5)}, {ItNumSetting(47,6)}}},
        {"P2/m2/n21/a", {{ItNumSetting(53,1)}}},
        {"P2/m21/a2/n", {{ItNumSetting(53,6)}}},
        {"P2/m21/c21/b", {{ItNumSetting(55,3)}, {ItNumSetting(55,4)}}},
        {"P2/m21/n21/n", {{ItNumSetting(58,3)}, {ItNumSetting(58,4)}}},
        {"P2/n-3", {{ItNumSetting(201,1)}, {ItNumSetting(201,2)}}},
        {"P2/n11", {{ItNumSetting(13,11)}, {ItNumSetting(13,12)}}},
        {"P2/n2/c2/b", {{ItNumSetting(50,5)}, {ItNumSetting(50,6)}, {ItNumSetting(50,7)}, {ItNumSetting(50,8)}}},
        {"P2/n2/m21/b", {{ItNumSetting(53,2)}}},
        {"P2/n2/n2/n", {{ItNumSetting(48,1)}, {ItNumSetting(48,2)}, {ItNumSetting(48,3)}, {ItNumSetting(48,4)}, {ItNumSetting(48,5)}, {ItNumSetting(48,6)}, {ItNumSetting(48,7)}, {ItNumSetting(48,8)}, {ItNumSetting(48,9)}, {ItNumSetting(48,10)}, {ItNumSetting(48,11)}, {ItNumSetting(48,12)}}},
        {"P2/n21/a21/a", {{ItNumSetting(56,3)}, {ItNumSetting(56,4)}}},
        {"P2/n21/c2/m", {{ItNumSetting(53,5)}}},
        {"P2/n21/m21/m", {{ItNumSetting(59,5)}, {ItNumSetting(59,6)}, {ItNumSetting(59,7)}, {ItNumSetting(59,8)}}},
        {"P2/n21/n2/a", {{ItNumSetting(52,1)}}},
        {"P2/n21/n2/b", {{ItNumSetting(52,2)}}},
        {"P211", {{ItNumSetting(3,5)}, {ItNumSetting(3,6)}}},
        {"P222", {{ItNumSetting(16,1)}, {ItNumSetting(16,2)}, {ItNumSetting(16,3)}, {ItNumSetting(16,4)}, {ItNumSetting(16,5)}, {ItNumSetting(16,6)}}},
        {"P2221", {{ItNumSetting(17,1)}, {ItNumSetting(17,2)}}},
        {"P2212", {{ItNumSetting(17,5)}, {ItNumSetting(17,6)}}},
        {"P22121", {{ItNumSetting(18,3)}, {ItNumSetting(18,4)}}},
        {"P23", {{ItNumSetting(195,1)}}},
        {"P21/a-3", {{ItNumSetting(205,1)}}},
        {"P21/b11", {{ItNumSetting(14,5)}, {ItNumSetting(14,18)}}},
        {"P21/b2/c21/n", {{ItNumSetting(60,1)}}},
        {"P21/b2/m2/n", {{ItNumSetting(53,3)}}},
        {"P21/b2/n21/b", {{ItNumSetting(56,5)}, {ItNumSetting(56,6)}}},
        {"P21/b21/a2/m", {{ItNumSetting(55,1)}, {ItNumSetting(55,2)}}},
        {"P21/b21/c21/a", {{ItNumSetting(61,1)}, {ItNumSetting(61,3)}, {ItNumSetting(61,5)}}},
        {"P21/b21/m2/a", {{ItNumSetting(57,5)}}},
        {"P21/b21/n21/m", {{ItNumSetting(62,3)}}},
        {"P21/c11", {{ItNumSetting(14,6)}, {ItNumSetting(14,17)}}},
        {"P21/c2/a21/n", {{ItNumSetting(60,2)}}},
        {"P21/c2/c2/a", {{ItNumSetting(54,1)}}},
        {"P21/c2/c2/b", {{ItNumSetting(54,2)}}},
        {"P21/c2/m21/a", {{ItNumSetting(55,5)}, {ItNumSetting(55,6)}}},
        {"P21/c2/n2/m", {{ItNumSetting(53,4)}}},
        {"P21/c21/a21/b", {{ItNumSetting(61,2)}, {ItNumSetting(61,4)}, {ItNumSetting(61,6)}}},
        {"P21/c21/c2/n", {{ItNumSetting(56,1)}, {ItNumSetting(56,2)}}},
        {"P21/c21/m2/b", {{ItNumSetting(57,6)}}},
        {"P21/c21/m21/n", {{ItNumSetting(62,4)}}},
        {"P21/m11", {{ItNumSetting(11,5)}, {ItNumSetting(11,6)}}},
        {"P21/m2/a21/b", {{ItNumSetting(57,4)}}},
        {"P21/m2/c21a", {{ItNumSetting(57,3)}}},
        {"P21/m2/m2/a", {{ItNumSetting(51,1)}}},
        {"P21/m2/m2/b", {{ItNumSetting(51,2)}}},
        {"P21/m2/n21/m", {{ItNumSetting(59,9)}, {ItNumSetting(59,11)}, {ItNumSetting(59,12)}}},
        {"P21/m2/n21m", {{ItNumSetting(59,10)}}},
        {"P21/m21/c21/n", {{ItNumSetting(62,5)}}},
        {"P21/m21/m2/n", {{ItNumSetting(59,1)}, {ItNumSetting(59,2)}, {ItNumSetting(59,3)}, {ItNumSetting(59,4)}}},
        {"P21/m21/n21/b", {{ItNumSetting(62,2)}}},
        {"P21/n11", {{ItNumSetting(14,11)}, {ItNumSetting(14,12)}}},
        {"P21/n2/a2/n", {{ItNumSetting(52,6)}}},
        {"P21/n2/c2/n", {{ItNumSetting(52,5)}}},
        {"P21/n2/m21/n", {{ItNumSetting(58,5)}, {ItNumSetting(58,6)}}},
        {"P21/n21/a2/b", {{ItNumSetting(60,4)}}},
        {"P21/n21/a21/m", {{ItNumSetting(62,6)}}},
        {"P21/n21/c2/a", {{ItNumSetting(60,3)}}},
        {"P21/n21/m21/a", {{ItNumSetting(62,1)}}},
        {"P21/n21/n2/m", {{ItNumSetting(58,1)}, {ItNumSetting(58,2)}}},
        {"P2111", {{ItNumSetting(4,5)}, {ItNumSetting(4,6)}}},
        {"P2122", {{ItNumSetting(17,3)}, {ItNumSetting(17,4)}}},
        {"P21221", {{ItNumSetting(18,5)}, {ItNumSetting(18,6)}}},
        {"P21212", {{ItNumSetting(18,1)}, {ItNumSetting(18,2)}}},
        {"P212121", {{ItNumSetting(19,1)}, {ItNumSetting(19,2)}, {ItNumSetting(19,3)}, {ItNumSetting(19,4)}, {ItNumSetting(19,5)}, {ItNumSetting(19,6)}}},
        {"P213", {{ItNumSetting(198,1)}}},
        {"P21ab", {{ItNumSetting(29,3)}}},
        {"P21am", {{ItNumSetting(26,4)}}},
        {"P21ca", {{ItNumSetting(29,4)}}},
        {"P21cn", {{ItNumSetting(33,4)}}},
        {"P21ma", {{ItNumSetting(26,3)}}},
        {"P21mn", {{ItNumSetting(31,3)}}},
        {"P21nb", {{ItNumSetting(33,3)}}},
        {"P21nm", {{ItNumSetting(31,4)}}},
        {"P2aa", {{ItNumSetting(27,3)}, {ItNumSetting(27,4)}}},
        {"P2an", {{ItNumSetting(30,4)}}},
        {"P2cb", {{ItNumSetting(32,3)}, {ItNumSetting(32,4)}}},
        {"P2cm", {{ItNumSetting(28,4)}}},
        {"P2mb", {{ItNumSetting(28,3)}}},
        {"P2mm", {{ItNumSetting(25,3)}, {ItNumSetting(25,4)}}},
        {"P2na", {{ItNumSetting(30,3)}}},
        {"P2nn", {{ItNumSetting(34,3)}, {ItNumSetting(34,4)}}},
        {"P3", {{ItNumSetting(143,1)}}},
        {"P312", {{ItNumSetting(149,1)}}},
        {"P31c", {{ItNumSetting(159,1)}}},
        {"P31m", {{ItNumSetting(157,1)}}},
        {"P321", {{ItNumSetting(150,1)}}},
        {"P31", {{ItNumSetting(144,1)}}},
        {"P3112", {{ItNumSetting(151,1)}}},
        {"P3121", {{ItNumSetting(152,1)}}},
        {"P32", {{ItNumSetting(145,1)}}},
        {"P3212", {{ItNumSetting(153,1)}}},
        {"P3221", {{ItNumSetting(154,1)}}},
        {"P3c1", {{ItNumSetting(158,1)}}},
        {"P3m1", {{ItNumSetting(156,1)}}},
        {"P4", {{ItNumSetting(75,1)}}},
        {"P4/m", {{ItNumSetting(83,1)}}},
        {"P4/m-32/m", {{ItNumSetting(221,1)}}},
        {"P4/m2/c2/c", {{ItNumSetting(124,1)}}},
        {"P4/m2/m2/m", {{ItNumSetting(123,1)}}},
        {"P4/m21/b2/m", {{ItNumSetting(127,1)}}},
        {"P4/m21/n2/c", {{ItNumSetting(128,1)}}},
        {"P4/n", {{ItNumSetting(85,1)}, {ItNumSetting(85,2)}}},
        {"P4/n-32/n", {{ItNumSetting(222,1)}, {ItNumSetting(222,2)}}},
        {"P4/n2/b2/m", {{ItNumSetting(125,1)}, {ItNumSetting(125,2)}}},
        {"P4/n2/n2/c", {{ItNumSetting(126,1)}, {ItNumSetting(126,2)}}},
        {"P4/n21/c2/c", {{ItNumSetting(130,1)}, {ItNumSetting(130,2)}}},
        {"P4/n21/m2/m", {{ItNumSetting(129,1)}, {ItNumSetting(129,2)}}},
        {"P422", {{ItNumSetting(89,1)}}},
        {"P4212", {{ItNumSetting(90,1)}}},
        {"P432", {{ItNumSetting(207,1)}}},
        {"P41", {{ItNumSetting(76,1)}}},
        {"P4122", {{ItNumSetting(91,1)}}},
        {"P41212", {{ItNumSetting(92,1)}}},
        {"P4132", {{ItNumSetting(213,1)}}},
        {"P42", {{ItNumSetting(77,1)}}},
        {"P42/m", {{ItNumSetting(84,1)}}},
        {"P42/m-32/n", {{ItNumSetting(223,1)}}},
        {"P42/m2/c2/m", {{ItNumSetting(132,1)}}},
        {"P42/m2/m2/c", {{ItNumSetting(131,1)}}},
        {"P42/m21/b2/c", {{ItNumSetting(135,1)}}},
        {"P42/m21/n2/m", {{ItNumSetting(136,1)}}},
        {"P42/n", {{ItNumSetting(86,1)}, {ItNumSetting(86,2)}}},
        {"P42/n-32/m", {{ItNumSetting(224,1)}, {ItNumSetting(224,2)}}},
        {"P42/n2/b2/c", {{ItNumSetting(133,1)}, {ItNumSetting(133,2)}}},
        {"P42/n2/n2/m", {{ItNumSetting(134,1)}, {ItNumSetting(134,2)}}},
        {"P42/n21/c2/m", {{ItNumSetting(138,1)}, {ItNumSetting(138,2)}}},
        {"P42/n21/m2/c", {{ItNumSetting(137,1)}, {ItNumSetting(137,2)}}},
        {"P4222", {{ItNumSetting(93,1)}}},
        {"P42212", {{ItNumSetting(94,1)}}},
        {"P4232", {{ItNumSetting(208,1)}}},
        {"P42bc", {{ItNumSetting(106,1)}}},
        {"P42cm", {{ItNumSetting(101,1)}}},
        {"P42mc", {{ItNumSetting(105,1)}}},
        {"P42nm", {{ItNumSetting(102,1)}}},
        {"P43", {{ItNumSetting(78,1)}}},
        {"P4322", {{ItNumSetting(95,1)}}},
        {"P43212", {{ItNumSetting(96,1)}}},
        {"P4332", {{ItNumSetting(212,1)}}},
        {"P4bm", {{ItNumSetting(100,1)}}},
        {"P4cc", {{ItNumSetting(103,1)}}},
        {"P4mm", {{ItNumSetting(99,1)}}},
        {"P4nc", {{ItNumSetting(104,1)}}},
        {"P6", {{ItNumSetting(168,1)}}},
        {"P6/m", {{ItNumSetting(175,1)}}},
        {"P6/m2/c2/c", {{ItNumSetting(192,1)}}},
        {"P6/m2/m2/m", {{ItNumSetting(191,1)}}},
        {"P622", {{ItNumSetting(177,1)}}},
        {"P61", {{ItNumSetting(169,1)}}},
        {"P6122", {{ItNumSetting(178,1)}}},
        {"P62", {{ItNumSetting(171,1)}}},
        {"P6222", {{ItNumSetting(180,1)}}},
        {"P63", {{ItNumSetting(173,1)}}},
        {"P63/m", {{ItNumSetting(176,1)}}},
        {"P63/m2/c2/m", {{ItNumSetting(193,1)}}},
        {"P63/m2/m2/c", {{ItNumSetting(194,1)}}},
        {"P6322", {{ItNumSetting(182,1)}}},
        {"P63cm", {{ItNumSetting(185,1)}}},
        {"P63mc", {{ItNumSetting(186,1)}}},
        {"P64", {{ItNumSetting(172,1)}}},
        {"P6422", {{ItNumSetting(181,1)}}},
        {"P65", {{ItNumSetting(170,1)}}},
        {"P6522", {{ItNumSetting(179,1)}}},
        {"P6cc", {{ItNumSetting(184,1)}}},
        {"P6mm", {{ItNumSetting(183,1)}}},
        {"Pb11", {{ItNumSetting(7,5)}, {ItNumSetting(7,18)}}},
        {"Pb21a", {{ItNumSetting(29,6)}}},
        {"Pb21m", {{ItNumSetting(26,5)}}},
        {"Pb2b", {{ItNumSetting(27,5)}, {ItNumSetting(27,6)}}},
        {"Pb2n", {{ItNumSetting(30,5)}}},
        {"Pba2", {{ItNumSetting(32,1)}, {ItNumSetting(32,2)}}},
        {"Pbc21", {{ItNumSetting(29,2)}}},
        {"Pbm2", {{ItNumSetting(28,2)}}},
        {"Pbn21", {{ItNumSetting(33,2)}}},
        {"Pc11", {{ItNumSetting(7,6)}, {ItNumSetting(7,17)}}},
        {"Pc21b", {{ItNumSetting(29,5)}}},
        {"Pc21n", {{ItNumSetting(33,5)}}},
        {"Pc2a", {{ItNumSetting(32,5)}, {ItNumSetting(32,6)}}},
        {"Pc2m", {{ItNumSetting(28,5)}}},
        {"Pca21", {{ItNumSetting(29,1)}}},
        {"Pcc2", {{ItNumSetting(27,1)}, {ItNumSetting(27,2)}}},
        {"Pcm21", {{ItNumSetting(26,2)}}},
        {"Pcn2", {{ItNumSetting(30,2)}}},
        {"Pm11", {{ItNumSetting(6,5)}, {ItNumSetting(6,6)}}},
        {"Pm21b", {{ItNumSetting(26,6)}}},
        {"Pm21n", {{ItNumSetting(31,6)}}},
        {"Pm2a", {{ItNumSetting(28,6)}}},
        {"Pm2m", {{ItNumSetting(25,5)}, {ItNumSetting(25,6)}}},
        {"Pma2", {{ItNumSetting(28,1)}}},
        {"Pmc21", {{ItNumSetting(26,1)}}},
        {"Pmm2", {{ItNumSetting(25,1)}, {ItNumSetting(25,2)}}},
        {"Pmn21", {{ItNumSetting(31,1)}}},
        {"Pn11", {{ItNumSetting(7,11)}, {ItNumSetting(7,12)}}},
        {"Pn21a", {{ItNumSetting(33,6)}}},
        {"Pn21m", {{ItNumSetting(31,5)}}},
        {"Pn2b", {{ItNumSetting(30,6)}}},
        {"Pn2n", {{ItNumSetting(34,5)}, {ItNumSetting(34,6)}}},
        {"Pna21", {{ItNumSetting(33,1)}}},
        {"Pnc2", {{ItNumSetting(30,1)}}},
        {"Pnm21", {{ItNumSetting(31,2)}}},
        {"Pnn2", {{ItNumSetting(34,1)}, {ItNumSetting(34,2)}}},
        {"R-3", {{ItNumSetting(148,1)}, {ItNumSetting(148,2)}}},
        {"R-32/c", {{ItNumSetting(167,1)}, {ItNumSetting(167,2)}}},
        {"R-32/m", {{ItNumSetting(166,1)}, {ItNumSetting(166,2)}}},
        {"R3", {{ItNumSetting(146,1)}, {ItNumSetting(146,2)}}},
        {"R32", {{ItNumSetting(155,1)}, {ItNumSetting(155,2)}}},
        {"R3c", {{ItNumSetting(161,1)}, {ItNumSetting(161,2)}}},
        {"R3m", {{ItNumSetting(160,1)}, {ItNumSetting(160,2)}}}
    };

    // full HM symbols in BGMN format (screw axes as 2_1)
    const QMap<QString, QList<ItNumSetting> > hmBgmn {
        {"A-1", {{ItNumSetting(2,3)}}},
        {"A1", {{ItNumSetting(1,3)}}},
        {"A112", {{ItNumSetting(5,3)}, {ItNumSetting(5,10)}}},
        {"A112/a", {{ItNumSetting(15,3)}}},
        {"A112/m", {{ItNumSetting(12,3)}, {ItNumSetting(12,10)}}},
        {"A112/n", {{ItNumSetting(15,10)}}},
        {"A11a", {{ItNumSetting(9,3)}}},
        {"A11m", {{ItNumSetting(8,3)}, {ItNumSetting(8,10)}}},
        {"A11n", {{ItNumSetting(9,10)}}},
        {"A12/a1", {{ItNumSetting(15,2)}}},
        {"A12/m1", {{ItNumSetting(12,2)}, {ItNumSetting(12,7)}}},
        {"A12/n1", {{ItNumSetting(15,7)}}},
        {"A121", {{ItNumSetting(5,2)}, {ItNumSetting(5,7)}}},
        {"A1a1", {{ItNumSetting(9,2)}}},
        {"A1m1", {{ItNumSetting(8,2)}, {ItNumSetting(8,7)}}},
        {"A1n1", {{ItNumSetting(9,7)}}},
        {"A2/b2/a2/a", {{ItNumSetting(68,5)}, {ItNumSetting(68,6)}}},
        {"A2/b2/m2/m", {{ItNumSetting(67,3)}}},
        {"A2/c2/a2/a", {{ItNumSetting(68,7)}, {ItNumSetting(68,8)}}},
        {"A2/c2/m2/m", {{ItNumSetting(67,4)}}},
        {"A2/m2/a2/a", {{ItNumSetting(66,3)}, {ItNumSetting(66,4)}}},
        {"A2/m2/m2/m", {{ItNumSetting(65,3)}, {ItNumSetting(65,4)}}},
        {"A222", {{ItNumSetting(21,3)}, {ItNumSetting(21,4)}}},
        {"A2_1/b2/m2/a", {{ItNumSetting(64,3)}}},
        {"A2_1/c2/a2/m", {{ItNumSetting(64,4)}}},
        {"A2_1/m2/a2/m", {{ItNumSetting(63,4)}}},
        {"A2_1/m2/m2/a", {{ItNumSetting(63,3)}}},
        {"A2_122", {{ItNumSetting(20,3)}, {ItNumSetting(20,4)}}},
        {"A2_1am", {{ItNumSetting(36,4)}}},
        {"A2_1ma", {{ItNumSetting(36,3)}}},
        {"A2aa", {{ItNumSetting(37,3)}, {ItNumSetting(37,4)}}},
        {"A2mm", {{ItNumSetting(35,3)}, {ItNumSetting(35,4)}}},
        {"Aba2", {{ItNumSetting(41,1)}}},
        {"Abm2", {{ItNumSetting(39,1)}}},
        {"Ac2a", {{ItNumSetting(41,6)}}},
        {"Ac2m", {{ItNumSetting(39,6)}}},
        {"Am2a", {{ItNumSetting(40,6)}}},
        {"Am2m", {{ItNumSetting(38,6)}}},
        {"Ama2", {{ItNumSetting(40,1)}}},
        {"Amm2", {{ItNumSetting(38,1)}}},
        {"B-1", {{ItNumSetting(2,4)}}},
        {"B1", {{ItNumSetting(1,4)}}},
        {"B112", {{ItNumSetting(5,4)}, {ItNumSetting(5,9)}}},
        {"B112/b", {{ItNumSetting(15,4)}}},
        {"B112/m", {{ItNumSetting(12,4)}, {ItNumSetting(12,9)}}},
        {"B112/n", {{ItNumSetting(15,9)}}},
        {"B11b", {{ItNumSetting(9,4)}}},
        {"B11m", {{ItNumSetting(8,4)}, {ItNumSetting(8,9)}}},
        {"B11n", {{ItNumSetting(9,9)}}},
        {"B2/b11", {{ItNumSetting(15,5)}}},
        {"B2/b2/a2/b", {{ItNumSetting(68,11)}, {ItNumSetting(68,12)}}},
        {"B2/b2/c2/b", {{ItNumSetting(68,9)}, {ItNumSetting(68,10)}}},
        {"B2/b2/m2/b", {{ItNumSetting(66,5)}, {ItNumSetting(66,6)}}},
        {"B2/b2_1/c2/m", {{ItNumSetting(64,5)}}},
        {"B2/b2_1/m2/m", {{ItNumSetting(63,5)}}},
        {"B2/m11", {{ItNumSetting(12,5)}, {ItNumSetting(12,12)}}},
        {"B2/m2/a2/m", {{ItNumSetting(67,6)}}},
        {"B2/m2/c2/m", {{ItNumSetting(67,5)}}},
        {"B2/m2/m2/m", {{ItNumSetting(65,5)}, {ItNumSetting(65,6)}}},
        {"B2/m2_1/a2/b", {{ItNumSetting(64,6)}}},
        {"B2/m2_1/m2/b", {{ItNumSetting(63,6)}}},
        {"B2/n11", {{ItNumSetting(15,12)}}},
        {"B211", {{ItNumSetting(5,5)}, {ItNumSetting(5,12)}}},
        {"B222", {{ItNumSetting(21,5)}, {ItNumSetting(21,6)}}},
        {"B22_12", {{ItNumSetting(20,5)}, {ItNumSetting(20,6)}}},
        {"B2cb", {{ItNumSetting(41,3)}}},
        {"B2cm", {{ItNumSetting(39,3)}}},
        {"B2mb", {{ItNumSetting(40,3)}}},
        {"B2mm", {{ItNumSetting(38,3)}}},
        {"Bb11", {{ItNumSetting(9,5)}}},
        {"Bb2_1m", {{ItNumSetting(36,5)}}},
        {"Bb2b", {{ItNumSetting(37,5)}, {ItNumSetting(37,6)}}},
        {"Bba2", {{ItNumSetting(41,2)}}},
        {"Bbm2", {{ItNumSetting(40,2)}}},
        {"Bm11", {{ItNumSetting(8,5)}, {ItNumSetting(8,12)}}},
        {"Bm2_1b", {{ItNumSetting(36,6)}}},
        {"Bm2m", {{ItNumSetting(35,5)}, {ItNumSetting(35,6)}}},
        {"Bma2", {{ItNumSetting(39,2)}}},
        {"Bmm2", {{ItNumSetting(38,2)}}},
        {"Bn11", {{ItNumSetting(9,12)}}},
        {"C-1", {{ItNumSetting(2,2)}}},
        {"C1", {{ItNumSetting(1,2)}}},
        {"C12/c1", {{ItNumSetting(15,1)}}},
        {"C12/m1", {{ItNumSetting(12,1)}, {ItNumSetting(12,8)}}},
        {"C12/n1", {{ItNumSetting(15,8)}}},
        {"C121", {{ItNumSetting(5,1)}, {ItNumSetting(5,8)}}},
        {"C1c1", {{ItNumSetting(9,1)}}},
        {"C1m1", {{ItNumSetting(8,1)}, {ItNumSetting(8,8)}}},
        {"C1n1", {{ItNumSetting(9,8)}}},
        {"C2/c11", {{ItNumSetting(15,6)}}},
        {"C2/c2/c2/a", {{ItNumSetting(68,1)}, {ItNumSetting(68,2)}}},
        {"C2/c2/c2/b", {{ItNumSetting(68,3)}, {ItNumSetting(68,4)}}},
        {"C2/c2/c2/m", {{ItNumSetting(66,1)}, {ItNumSetting(66,2)}}},
        {"C2/c2/m2_1/b", {{ItNumSetting(64,2)}}},
        {"C2/c2/m2_1m", {{ItNumSetting(63,2)}}},
        {"C2/m11", {{ItNumSetting(12,6)}, {ItNumSetting(12,11)}}},
        {"C2/m2/c2_1/a", {{ItNumSetting(64,1)}}},
        {"C2/m2/c2_1/m", {{ItNumSetting(63,1)}}},
        {"C2/m2/m2/a", {{ItNumSetting(67,1)}}},
        {"C2/m2/m2/b", {{ItNumSetting(67,2)}}},
        {"C2/m2/m2/m", {{ItNumSetting(65,1)}, {ItNumSetting(65,2)}}},
        {"C2/n11", {{ItNumSetting(15,11)}}},
        {"C211", {{ItNumSetting(5,6)}, {ItNumSetting(5,11)}}},
        {"C222", {{ItNumSetting(21,1)}, {ItNumSetting(21,2)}}},
        {"C222_1", {{ItNumSetting(20,1)}, {ItNumSetting(20,2)}}},
        {"C2cb", {{ItNumSetting(41,4)}}},
        {"C2cm", {{ItNumSetting(40,4)}}},
        {"C2mb", {{ItNumSetting(39,4)}}},
        {"C2mm", {{ItNumSetting(38,4)}}},
        {"Cc11", {{ItNumSetting(9,6)}}},
        {"Cc2a", {{ItNumSetting(41,5)}}},
        {"Cc2m", {{ItNumSetting(40,5)}}},
        {"Ccc2", {{ItNumSetting(37,1)}, {ItNumSetting(37,2)}}},
        {"Ccm2_1", {{ItNumSetting(36,2)}}},
        {"Cm11", {{ItNumSetting(8,6)}, {ItNumSetting(8,11)}}},
        {"Cm2a", {{ItNumSetting(39,5)}}},
        {"Cm2m", {{ItNumSetting(38,5)}}},
        {"Cmc2_1", {{ItNumSetting(36,1)}}},
        {"Cmm2", {{ItNumSetting(35,1)}, {ItNumSetting(35,2)}}},
        {"Cn11", {{ItNumSetting(9,11)}}},
        {"F-1", {{ItNumSetting(2,6)}}},
        {"F-43c", {{ItNumSetting(219,1)}}},
        {"F-43m", {{ItNumSetting(216,1)}}},
        {"F1", {{ItNumSetting(1,6)}}},
        {"F121", {{ItNumSetting(3,7)}}},
        {"F2/d-3", {{ItNumSetting(203,1)}, {ItNumSetting(203,2)}}},
        {"F2/d2/d2/d", {{ItNumSetting(70,1)}, {ItNumSetting(70,2)}, {ItNumSetting(70,3)}, {ItNumSetting(70,4)}, {ItNumSetting(70,5)}, {ItNumSetting(70,6)}, {ItNumSetting(70,7)}, {ItNumSetting(70,8)}, {ItNumSetting(70,9)}, {ItNumSetting(70,10)}, {ItNumSetting(70,11)}, {ItNumSetting(70,12)}}},
        {"F2/m-3", {{ItNumSetting(202,1)}}},
        {"F2/m2/m2/m", {{ItNumSetting(69,1)}, {ItNumSetting(69,2)}, {ItNumSetting(69,3)}, {ItNumSetting(69,4)}, {ItNumSetting(69,5)}, {ItNumSetting(69,6)}}},
        {"F222", {{ItNumSetting(22,1)}, {ItNumSetting(22,2)}, {ItNumSetting(22,3)}, {ItNumSetting(22,4)}, {ItNumSetting(22,5)}, {ItNumSetting(22,6)}}},
        {"F23", {{ItNumSetting(196,1)}}},
        {"F2dd", {{ItNumSetting(43,3)}, {ItNumSetting(43,4)}}},
        {"F2mm", {{ItNumSetting(42,3)}, {ItNumSetting(42,4)}}},
        {"F4/m-32/c", {{ItNumSetting(226,1)}}},
        {"F4/m-32/m", {{ItNumSetting(225,1)}}},
        {"F432", {{ItNumSetting(209,1)}}},
        {"F4_1/d-32/c", {{ItNumSetting(228,1)}, {ItNumSetting(228,2)}}},
        {"F4_1/d-32/m", {{ItNumSetting(227,1)}, {ItNumSetting(227,2)}}},
        {"F4_132", {{ItNumSetting(210,1)}}},
        {"Fd2d", {{ItNumSetting(43,5)}, {ItNumSetting(43,6)}}},
        {"Fdd2", {{ItNumSetting(43,1)}, {ItNumSetting(43,2)}}},
        {"Fm2m", {{ItNumSetting(42,5)}, {ItNumSetting(42,6)}}},
        {"Fmm2", {{ItNumSetting(42,1)}, {ItNumSetting(42,2)}}},
        {"I-1", {{ItNumSetting(2,5)}}},
        {"I-4", {{ItNumSetting(82,1)}}},
        {"I-42d", {{ItNumSetting(122,1)}}},
        {"I-42m", {{ItNumSetting(121,1)}}},
        {"I-43d", {{ItNumSetting(220,1)}}},
        {"I-43m", {{ItNumSetting(217,1)}}},
        {"I-4c2", {{ItNumSetting(120,1)}}},
        {"I-4m2", {{ItNumSetting(119,1)}}},
        {"I1", {{ItNumSetting(1,5)}}},
        {"I112", {{ItNumSetting(5,15)}, {ItNumSetting(5,16)}}},
        {"I112/a", {{ItNumSetting(15,16)}}},
        {"I112/b", {{ItNumSetting(15,15)}}},
        {"I112/m", {{ItNumSetting(12,15)}, {ItNumSetting(12,16)}}},
        {"I11a", {{ItNumSetting(9,16)}}},
        {"I11b", {{ItNumSetting(9,15)}}},
        {"I11m", {{ItNumSetting(8,15)}, {ItNumSetting(8,16)}}},
        {"I12/a1", {{ItNumSetting(15,13)}}},
        {"I12/c1", {{ItNumSetting(15,14)}}},
        {"I12/m1", {{ItNumSetting(12,13)}, {ItNumSetting(12,14)}}},
        {"I121", {{ItNumSetting(5,13)}, {ItNumSetting(5,14)}}},
        {"I1a1", {{ItNumSetting(9,13)}}},
        {"I1c1", {{ItNumSetting(9,14)}}},
        {"I1m1", {{ItNumSetting(8,13)}, {ItNumSetting(8,14)}}},
        {"I2/b11", {{ItNumSetting(15,18)}}},
        {"I2/b2/a2/m", {{ItNumSetting(72,1)}, {ItNumSetting(72,2)}}},
        {"I2/c11", {{ItNumSetting(15,17)}}},
        {"I2/c2/m2/a", {{ItNumSetting(72,5)}, {ItNumSetting(72,6)}}},
        {"I2/m-3", {{ItNumSetting(204,1)}}},
        {"I2/m11", {{ItNumSetting(12,17)}, {ItNumSetting(12,18)}}},
        {"I2/m2/c2/b", {{ItNumSetting(72,3)}, {ItNumSetting(72,4)}}},
        {"I2/m2/m2/m", {{ItNumSetting(71,1)}, {ItNumSetting(71,2)}, {ItNumSetting(71,3)}, {ItNumSetting(71,4)}, {ItNumSetting(71,5)}, {ItNumSetting(71,6)}}},
        {"I211", {{ItNumSetting(5,17)}, {ItNumSetting(5,18)}}},
        {"I222", {{ItNumSetting(23,1)}, {ItNumSetting(23,2)}, {ItNumSetting(23,3)}, {ItNumSetting(23,4)}, {ItNumSetting(23,5)}, {ItNumSetting(23,6)}}},
        {"I23", {{ItNumSetting(197,1)}}},
        {"I2_1/a-3", {{ItNumSetting(206,1)}}},
        {"I2_1/b2_1/c2_1/a", {{ItNumSetting(73,1)}, {ItNumSetting(73,3)}, {ItNumSetting(73,5)}}},
        {"I2_1/b2_1/m2_1/m", {{ItNumSetting(74,3)}}},
        {"I2_1/c2_1/a2_1/b", {{ItNumSetting(73,2)}, {ItNumSetting(73,4)}, {ItNumSetting(73,6)}}},
        {"I2_1/c2_1/m2_1/m", {{ItNumSetting(74,4)}}},
        {"I2_1/m2_1/a2_1/m", {{ItNumSetting(74,6)}}},
        {"I2_1/m2_1/c2_1/m", {{ItNumSetting(74,5)}}},
        {"I2_1/m2_1/m2_1/a", {{ItNumSetting(74,1)}}},
        {"I2_1/m2_1/m2_1/b", {{ItNumSetting(74,2)}}},
        {"I2_12_12_1", {{ItNumSetting(24,1)}, {ItNumSetting(24,2)}, {ItNumSetting(24,3)}, {ItNumSetting(24,4)}, {ItNumSetting(24,5)}, {ItNumSetting(24,6)}}},
        {"I2_13", {{ItNumSetting(199,1)}}},
        {"I2cb", {{ItNumSetting(45,3)}, {ItNumSetting(45,4)}}},
        {"I2cm", {{ItNumSetting(46,4)}}},
        {"I2mb", {{ItNumSetting(46,3)}}},
        {"I2mm", {{ItNumSetting(44,3)}, {ItNumSetting(44,4)}}},
        {"I4", {{ItNumSetting(79,1)}}},
        {"I4/m", {{ItNumSetting(87,1)}}},
        {"I4/m-32/m", {{ItNumSetting(229,1)}}},
        {"I4/m2/c2/m", {{ItNumSetting(140,1)}}},
        {"I4/m2/m2/m", {{ItNumSetting(139,1)}}},
        {"I422", {{ItNumSetting(97,1)}}},
        {"I432", {{ItNumSetting(211,1)}}},
        {"I4_1", {{ItNumSetting(80,1)}}},
        {"I4_1/a", {{ItNumSetting(88,1)}, {ItNumSetting(88,2)}}},
        {"I4_1/a-32/d", {{ItNumSetting(230,1)}}},
        {"I4_1/a2/c2/d", {{ItNumSetting(142,1)}, {ItNumSetting(142,2)}}},
        {"I4_1/a2/m2/d", {{ItNumSetting(141,1)}, {ItNumSetting(141,2)}}},
        {"I4_122", {{ItNumSetting(98,1)}}},
        {"I4_132", {{ItNumSetting(214,1)}}},
        {"I4_1cd", {{ItNumSetting(110,1)}}},
        {"I4_1md", {{ItNumSetting(109,1)}}},
        {"I4cm", {{ItNumSetting(108,1)}}},
        {"I4mm", {{ItNumSetting(107,1)}}},
        {"Ib11", {{ItNumSetting(9,18)}}},
        {"Iba2", {{ItNumSetting(45,1)}, {ItNumSetting(45,2)}}},
        {"Ibm2", {{ItNumSetting(46,2)}}},
        {"Ic11", {{ItNumSetting(9,17)}}},
        {"Ic2a", {{ItNumSetting(45,5)}, {ItNumSetting(45,6)}}},
        {"Ic2m", {{ItNumSetting(46,5)}}},
        {"Im11", {{ItNumSetting(8,17)}, {ItNumSetting(8,18)}}},
        {"Im2a", {{ItNumSetting(46,6)}}},
        {"Im2m", {{ItNumSetting(44,5)}, {ItNumSetting(44,6)}}},
        {"Ima2", {{ItNumSetting(46,1)}}},
        {"Imm2", {{ItNumSetting(44,1)}, {ItNumSetting(44,2)}}},
        {"P-1", {{ItNumSetting(2,1)}}},
        {"P-3", {{ItNumSetting(147,1)}}},
        {"P-312/c", {{ItNumSetting(163,1)}}},
        {"P-312/m", {{ItNumSetting(162,1)}}},
        {"P-32/c1", {{ItNumSetting(165,1)}}},
        {"P-32/m1", {{ItNumSetting(164,1)}}},
        {"P-4", {{ItNumSetting(81,1)}}},
        {"P-42_1c", {{ItNumSetting(114,1)}}},
        {"P-42_1m", {{ItNumSetting(113,1)}}},
        {"P-42c", {{ItNumSetting(112,1)}}},
        {"P-42m", {{ItNumSetting(111,1)}}},
        {"P-43m", {{ItNumSetting(215,1)}}},
        {"P-43n", {{ItNumSetting(218,1)}}},
        {"P-4b2", {{ItNumSetting(117,1)}}},
        {"P-4c2", {{ItNumSetting(116,1)}}},
        {"P-4m2", {{ItNumSetting(115,1)}}},
        {"P-4n2", {{ItNumSetting(118,1)}}},
        {"P-6", {{ItNumSetting(174,1)}}},
        {"P-62c", {{ItNumSetting(190,1)}}},
        {"P-62m", {{ItNumSetting(189,1)}}},
        {"P-6c2", {{ItNumSetting(188,1)}}},
        {"P-6m2", {{ItNumSetting(187,1)}}},
        {"P1", {{ItNumSetting(1,1)}}},
        {"P112", {{ItNumSetting(3,3)}, {ItNumSetting(3,4)}}},
        {"P112/a", {{ItNumSetting(13,3)}, {ItNumSetting(13,16)}}},
        {"P112/b", {{ItNumSetting(13,4)}, {ItNumSetting(13,15)}}},
        {"P112/m", {{ItNumSetting(10,3)}, {ItNumSetting(10,4)}}},
        {"P112/n", {{ItNumSetting(13,9)}, {ItNumSetting(13,10)}}},
        {"P112_1", {{ItNumSetting(4,3)}, {ItNumSetting(4,4)}}},
        {"P112_1/a", {{ItNumSetting(14,3)}, {ItNumSetting(14,16)}}},
        {"P112_1/b", {{ItNumSetting(14,4)}, {ItNumSetting(14,15)}}},
        {"P112_1/m", {{ItNumSetting(11,3)}, {ItNumSetting(11,4)}}},
        {"P112_1/n", {{ItNumSetting(14,9)}, {ItNumSetting(14,10)}}},
        {"P11a", {{ItNumSetting(7,3)}, {ItNumSetting(7,16)}}},
        {"P11b", {{ItNumSetting(7,4)}, {ItNumSetting(7,15)}}},
        {"P11m", {{ItNumSetting(6,3)}, {ItNumSetting(6,4)}}},
        {"P11n", {{ItNumSetting(7,9)}, {ItNumSetting(7,10)}}},
        {"P12/a1", {{ItNumSetting(13,2)}, {ItNumSetting(13,13)}}},
        {"P12/c1", {{ItNumSetting(13,1)}, {ItNumSetting(13,14)}}},
        {"P12/m1", {{ItNumSetting(10,1)}, {ItNumSetting(10,2)}}},
        {"P12/n1", {{ItNumSetting(13,7)}, {ItNumSetting(13,8)}}},
        {"P121", {{ItNumSetting(3,1)}, {ItNumSetting(3,2)}}},
        {"P12_1/a1", {{ItNumSetting(14,2)}, {ItNumSetting(14,13)}}},
        {"P12_1/c1", {{ItNumSetting(14,1)}, {ItNumSetting(14,14)}}},
        {"P12_1/m1", {{ItNumSetting(11,1)}, {ItNumSetting(11,2)}}},
        {"P12_1/n1", {{ItNumSetting(14,7)}, {ItNumSetting(14,8)}}},
        {"P12_11", {{ItNumSetting(4,1)}, {ItNumSetting(4,2)}}},
        {"P1a1", {{ItNumSetting(7,2)}, {ItNumSetting(7,13)}}},
        {"P1c1", {{ItNumSetting(7,1)}, {ItNumSetting(7,14)}}},
        {"P1m1", {{ItNumSetting(6,1)}, {ItNumSetting(6,2)}}},
        {"P1n1", {{ItNumSetting(7,7)}, {ItNumSetting(7,8)}}},
        {"P2/b11", {{ItNumSetting(13,5)}, {ItNumSetting(13,18)}}},
        {"P2/b2/a2/n", {{ItNumSetting(50,1)}, {ItNumSetting(50,2)}, {ItNumSetting(50,3)}, {ItNumSetting(50,4)}}},
        {"P2/b2/a2_1/b", {{ItNumSetting(54,6)}}},
        {"P2/b2/c2_1/b", {{ItNumSetting(54,5)}}},
        {"P2/b2/m2/b", {{ItNumSetting(49,5)}, {ItNumSetting(49,6)}}},
        {"P2/b2/n2_1/n", {{ItNumSetting(52,3)}}},
        {"P2/b2_1/a2/a", {{ItNumSetting(54,3)}}},
        {"P2/b2_1/c2_1/m", {{ItNumSetting(57,1)}}},
        {"P2/b2_1/m2/m", {{ItNumSetting(51,3)}}},
        {"P2/b2_1/n2_1/a", {{ItNumSetting(60,5)}}},
        {"P2/c11", {{ItNumSetting(13,6)}, {ItNumSetting(13,17)}}},
        {"P2/c2/c2/m", {{ItNumSetting(49,1)}, {ItNumSetting(49,2)}}},
        {"P2/c2/n2/a", {{ItNumSetting(50,9)}, {ItNumSetting(50,10)}, {ItNumSetting(50,11)}, {ItNumSetting(50,12)}}},
        {"P2/c2/n2_1/n", {{ItNumSetting(52,4)}}},
        {"P2/c2_1/a2/a", {{ItNumSetting(54,4)}}},
        {"P2/c2_1/a2_1/m", {{ItNumSetting(57,2)}}},
        {"P2/c2_1/m2/m", {{ItNumSetting(51,4)}}},
        {"P2/c2_1/n2_1/b", {{ItNumSetting(60,6)}}},
        {"P2/m-3", {{ItNumSetting(200,1)}}},
        {"P2/m11", {{ItNumSetting(10,5)}, {ItNumSetting(10,6)}}},
        {"P2/m2/a2/a", {{ItNumSetting(49,3)}, {ItNumSetting(49,4)}}},
        {"P2/m2/a2_1/m", {{ItNumSetting(51,6)}}},
        {"P2/m2/c2_1/m", {{ItNumSetting(51,5)}}},
        {"P2/m2/m2/m", {{ItNumSetting(47,1)}, {ItNumSetting(47,2)}, {ItNumSetting(47,3)}, {ItNumSetting(47,4)}, {ItNumSetting(47,5)}, {ItNumSetting(47,6)}}},
        {"P2/m2/n2_1/a", {{ItNumSetting(53,1)}}},
        {"P2/m2_1/a2/n", {{ItNumSetting(53,6)}}},
        {"P2/m2_1/c2_1/b", {{ItNumSetting(55,3)}, {ItNumSetting(55,4)}}},
        {"P2/m2_1/n2_1/n", {{ItNumSetting(58,3)}, {ItNumSetting(58,4)}}},
        {"P2/n-3", {{ItNumSetting(201,1)}, {ItNumSetting(201,2)}}},
        {"P2/n11", {{ItNumSetting(13,11)}, {ItNumSetting(13,12)}}},
        {"P2/n2/c2/b", {{ItNumSetting(50,5)}, {ItNumSetting(50,6)}, {ItNumSetting(50,7)}, {ItNumSetting(50,8)}}},
        {"P2/n2/m2_1/b", {{ItNumSetting(53,2)}}},
        {"P2/n2/n2/n", {{ItNumSetting(48,1)}, {ItNumSetting(48,2)}, {ItNumSetting(48,3)}, {ItNumSetting(48,4)}, {ItNumSetting(48,5)}, {ItNumSetting(48,6)}, {ItNumSetting(48,7)}, {ItNumSetting(48,8)}, {ItNumSetting(48,9)}, {ItNumSetting(48,10)}, {ItNumSetting(48,11)}, {ItNumSetting(48,12)}}},
        {"P2/n2_1/a2_1/a", {{ItNumSetting(56,3)}, {ItNumSetting(56,4)}}},
        {"P2/n2_1/c2/m", {{ItNumSetting(53,5)}}},
        {"P2/n2_1/m2_1/m", {{ItNumSetting(59,5)}, {ItNumSetting(59,6)}, {ItNumSetting(59,7)}, {ItNumSetting(59,8)}}},
        {"P2/n2_1/n2/a", {{ItNumSetting(52,1)}}},
        {"P2/n2_1/n2/b", {{ItNumSetting(52,2)}}},
        {"P211", {{ItNumSetting(3,5)}, {ItNumSetting(3,6)}}},
        {"P222", {{ItNumSetting(16,1)}, {ItNumSetting(16,2)}, {ItNumSetting(16,3)}, {ItNumSetting(16,4)}, {ItNumSetting(16,5)}, {ItNumSetting(16,6)}}},
        {"P222_1", {{ItNumSetting(17,1)}, {ItNumSetting(17,2)}}},
        {"P22_12", {{ItNumSetting(17,5)}, {ItNumSetting(17,6)}}},
        {"P22_12_1", {{ItNumSetting(18,3)}, {ItNumSetting(18,4)}}},
        {"P23", {{ItNumSetting(195,1)}}},
        {"P2_1/a-3", {{ItNumSetting(205,1)}}},
        {"P2_1/b11", {{ItNumSetting(14,5)}, {ItNumSetting(14,18)}}},
        {"P2_1/b2/c2_1/n", {{ItNumSetting(60,1)}}},
        {"P2_1/b2/m2/n", {{ItNumSetting(53,3)}}},
        {"P2_1/b2/n2_1/b", {{ItNumSetting(56,5)}, {ItNumSetting(56,6)}}},
        {"P2_1/b2_1/a2/m", {{ItNumSetting(55,1)}, {ItNumSetting(55,2)}}},
        {"P2_1/b2_1/c2_1/a", {{ItNumSetting(61,1)}, {ItNumSetting(61,3)}, {ItNumSetting(61,5)}}},
        {"P2_1/b2_1/m2/a", {{ItNumSetting(57,5)}}},
        {"P2_1/b2_1/n2_1/m", {{ItNumSetting(62,3)}}},
        {"P2_1/c11", {{ItNumSetting(14,6)}, {ItNumSetting(14,17)}}},
        {"P2_1/c2/a2_1/n", {{ItNumSetting(60,2)}}},
        {"P2_1/c2/c2/a", {{ItNumSetting(54,1)}}},
        {"P2_1/c2/c2/b", {{ItNumSetting(54,2)}}},
        {"P2_1/c2/m2_1/a", {{ItNumSetting(55,5)}, {ItNumSetting(55,6)}}},
        {"P2_1/c2/n2/m", {{ItNumSetting(53,4)}}},
        {"P2_1/c2_1/a2_1/b", {{ItNumSetting(61,2)}, {ItNumSetting(61,4)}, {ItNumSetting(61,6)}}},
        {"P2_1/c2_1/c2/n", {{ItNumSetting(56,1)}, {ItNumSetting(56,2)}}},
        {"P2_1/c2_1/m2/b", {{ItNumSetting(57,6)}}},
        {"P2_1/c2_1/m2_1/n", {{ItNumSetting(62,4)}}},
        {"P2_1/m11", {{ItNumSetting(11,5)}, {ItNumSetting(11,6)}}},
        {"P2_1/m2/a2_1/b", {{ItNumSetting(57,4)}}},
        {"P2_1/m2/c2_1a", {{ItNumSetting(57,3)}}},
        {"P2_1/m2/m2/a", {{ItNumSetting(51,1)}}},
        {"P2_1/m2/m2/b", {{ItNumSetting(51,2)}}},
        {"P2_1/m2/n2_1/m", {{ItNumSetting(59,9)}, {ItNumSetting(59,11)}, {ItNumSetting(59,12)}}},
        {"P2_1/m2/n2_1m", {{ItNumSetting(59,10)}}},
        {"P2_1/m2_1/c2_1/n", {{ItNumSetting(62,5)}}},
        {"P2_1/m2_1/m2/n", {{ItNumSetting(59,1)}, {ItNumSetting(59,2)}, {ItNumSetting(59,3)}, {ItNumSetting(59,4)}}},
        {"P2_1/m2_1/n2_1/b", {{ItNumSetting(62,2)}}},
        {"P2_1/n11", {{ItNumSetting(14,11)}, {ItNumSetting(14,12)}}},
        {"P2_1/n2/a2/n", {{ItNumSetting(52,6)}}},
        {"P2_1/n2/c2/n", {{ItNumSetting(52,5)}}},
        {"P2_1/n2/m2_1/n", {{ItNumSetting(58,5)}, {ItNumSetting(58,6)}}},
        {"P2_1/n2_1/a2/b", {{ItNumSetting(60,4)}}},
        {"P2_1/n2_1/a2_1/m", {{ItNumSetting(62,6)}}},
        {"P2_1/n2_1/c2/a", {{ItNumSetting(60,3)}}},
        {"P2_1/n2_1/m2_1/a", {{ItNumSetting(62,1)}}},
        {"P2_1/n2_1/n2/m", {{ItNumSetting(58,1)}, {ItNumSetting(58,2)}}},
        {"P2_111", {{ItNumSetting(4,5)}, {ItNumSetting(4,6)}}},
        {"P2_122", {{ItNumSetting(17,3)}, {ItNumSetting(17,4)}}},
        {"P2_122_1", {{ItNumSetting(18,5)}, {ItNumSetting(18,6)}}},
        {"P2_12_12", {{ItNumSetting(18,1)}, {ItNumSetting(18,2)}}},
        {"P2_12_12_1", {{ItNumSetting(19,1)}, {ItNumSetting(19,2)}, {ItNumSetting(19,3)}, {ItNumSetting(19,4)}, {ItNumSetting(19,5)}, {ItNumSetting(19,6)}}},
        {"P2_13", {{ItNumSetting(198,1)}}},
        {"P2_1ab", {{ItNumSetting(29,3)}}},
        {"P2_1am", {{ItNumSetting(26,4)}}},
        {"P2_1ca", {{ItNumSetting(29,4)}}},
        {"P2_1cn", {{ItNumSetting(33,4)}}},
        {"P2_1ma", {{ItNumSetting(26,3)}}},
        {"P2_1mn", {{ItNumSetting(31,3)}}},
        {"P2_1nb", {{ItNumSetting(33,3)}}},
        {"P2_1nm", {{ItNumSetting(31,4)}}},
        {"P2aa", {{ItNumSetting(27,3)}, {ItNumSetting(27,4)}}},
        {"P2an", {{ItNumSetting(30,4)}}},
        {"P2cb", {{ItNumSetting(32,3)}, {ItNumSetting(32,4)}}},
        {"P2cm", {{ItNumSetting(28,4)}}},
        {"P2mb", {{ItNumSetting(28,3)}}},
        {"P2mm", {{ItNumSetting(25,3)}, {ItNumSetting(25,4)}}},
        {"P2na", {{ItNumSetting(30,3)}}},
        {"P2nn", {{ItNumSetting(34,3)}, {ItNumSetting(34,4)}}},
        {"P3", {{ItNumSetting(143,1)}}},
        {"P312", {{ItNumSetting(149,1)}}},
        {"P31c", {{ItNumSetting(159,1)}}},
        {"P31m", {{ItNumSetting(157,1)}}},
        {"P321", {{ItNumSetting(150,1)}}},
        {"P3_1", {{ItNumSetting(144,1)}}},
        {"P3_112", {{ItNumSetting(151,1)}}},
        {"P3_121", {{ItNumSetting(152,1)}}},
        {"P3_2", {{ItNumSetting(145,1)}}},
        {"P3_212", {{ItNumSetting(153,1)}}},
        {"P3_221", {{ItNumSetting(154,1)}}},
        {"P3c1", {{ItNumSetting(158,1)}}},
        {"P3m1", {{ItNumSetting(156,1)}}},
        {"P4", {{ItNumSetting(75,1)}}},
        {"P4/m", {{ItNumSetting(83,1)}}},
        {"P4/m-32/m", {{ItNumSetting(221,1)}}},
        {"P4/m2/c2/c", {{ItNumSetting(124,1)}}},
        {"P4/m2/m2/m", {{ItNumSetting(123,1)}}},
        {"P4/m2_1/b2/m", {{ItNumSetting(127,1)}}},
        {"P4/m2_1/n2/c", {{ItNumSetting(128,1)}}},
        {"P4/n", {{ItNumSetting(85,1)}, {ItNumSetting(85,2)}}},
        {"P4/n-32/n", {{ItNumSetting(222,1)}, {ItNumSetting(222,2)}}},
        {"P4/n2/b2/m", {{ItNumSetting(125,1)}, {ItNumSetting(125,2)}}},
        {"P4/n2/n2/c", {{ItNumSetting(126,1)}, {ItNumSetting(126,2)}}},
        {"P4/n2_1/c2/c", {{ItNumSetting(130,1)}, {ItNumSetting(130,2)}}},
        {"P4/n2_1/m2/m", {{ItNumSetting(129,1)}, {ItNumSetting(129,2)}}},
        {"P422", {{ItNumSetting(89,1)}}},
        {"P42_12", {{ItNumSetting(90,1)}}},
        {"P432", {{ItNumSetting(207,1)}}},
        {"P4_1", {{ItNumSetting(76,1)}}},
        {"P4_122", {{ItNumSetting(91,1)}}},
        {"P4_12_12", {{ItNumSetting(92,1)}}},
        {"P4_132", {{ItNumSetting(213,1)}}},
        {"P4_2", {{ItNumSetting(77,1)}}},
        {"P4_2/m", {{ItNumSetting(84,1)}}},
        {"P4_2/m-32/n", {{ItNumSetting(223,1)}}},
        {"P4_2/m2/c2/m", {{ItNumSetting(132,1)}}},
        {"P4_2/m2/m2/c", {{ItNumSetting(131,1)}}},
        {"P4_2/m2_1/b2/c", {{ItNumSetting(135,1)}}},
        {"P4_2/m2_1/n2/m", {{ItNumSetting(136,1)}}},
        {"P4_2/n", {{ItNumSetting(86,1)}, {ItNumSetting(86,2)}}},
        {"P4_2/n-32/m", {{ItNumSetting(224,1)}, {ItNumSetting(224,2)}}},
        {"P4_2/n2/b2/c", {{ItNumSetting(133,1)}, {ItNumSetting(133,2)}}},
        {"P4_2/n2/n2/m", {{ItNumSetting(134,1)}, {ItNumSetting(134,2)}}},
        {"P4_2/n2_1/c2/m", {{ItNumSetting(138,1)}, {ItNumSetting(138,2)}}},
        {"P4_2/n2_1/m2/c", {{ItNumSetting(137,1)}, {ItNumSetting(137,2)}}},
        {"P4_222", {{ItNumSetting(93,1)}}},
        {"P4_22_12", {{ItNumSetting(94,1)}}},
        {"P4_232", {{ItNumSetting(208,1)}}},
        {"P4_2bc", {{ItNumSetting(106,1)}}},
        {"P4_2cm", {{ItNumSetting(101,1)}}},
        {"P4_2mc", {{ItNumSetting(105,1)}}},
        {"P4_2nm", {{ItNumSetting(102,1)}}},
        {"P4_3", {{ItNumSetting(78,1)}}},
        {"P4_322", {{ItNumSetting(95,1)}}},
        {"P4_32_12", {{ItNumSetting(96,1)}}},
        {"P4_332", {{ItNumSetting(212,1)}}},
        {"P4bm", {{ItNumSetting(100,1)}}},
        {"P4cc", {{ItNumSetting(103,1)}}},
        {"P4mm", {{ItNumSetting(99,1)}}},
        {"P4nc", {{ItNumSetting(104,1)}}},
        {"P6", {{ItNumSetting(168,1)}}},
        {"P6/m", {{ItNumSetting(175,1)}}},
        {"P6/m2/c2/c", {{ItNumSetting(192,1)}}},
        {"P6/m2/m2/m", {{ItNumSetting(191,1)}}},
        {"P622", {{ItNumSetting(177,1)}}},
        {"P6_1", {{ItNumSetting(169,1)}}},
        {"P6_122", {{ItNumSetting(178,1)}}},
        {"P6_2", {{ItNumSetting(171,1)}}},
        {"P6_222", {{ItNumSetting(180,1)}}},
        {"P6_3", {{ItNumSetting(173,1)}}},
        {"P6_3/m", {{ItNumSetting(176,1)}}},
        {"P6_3/m2/c2/m", {{ItNumSetting(193,1)}}},
        {"P6_3/m2/m2/c", {{ItNumSetting(194,1)}}},
        {"P6_322", {{ItNumSetting(182,1)}}},
        {"P6_3cm", {{ItNumSetting(185,1)}}},
        {"P6_3mc", {{ItNumSetting(186,1)}}},
        {"P6_4", {{ItNumSetting(172,1)}}},
        {"P6_422", {{ItNumSetting(181,1)}}},
        {"P6_5", {{ItNumSetting(170,1)}}},
        {"P6_522", {{ItNumSetting(179,1)}}},
        {"P6cc", {{ItNumSetting(184,1)}}},
        {"P6mm", {{ItNumSetting(183,1)}}},
        {"Pb11", {{ItNumSetting(7,5)}, {ItNumSetting(7,18)}}},
        {"Pb2_1a", {{ItNumSetting(29,6)}}},
        {"Pb2_1m", {{ItNumSetting(26,5)}}},
        {"Pb2b", {{ItNumSetting(27,5)}, {ItNumSetting(27,6)}}},
        {"Pb2n", {{ItNumSetting(30,5)}}},
        {"Pba2", {{ItNumSetting(32,1)}, {ItNumSetting(32,2)}}},
        {"Pbc2_1", {{ItNumSetting(29,2)}}},
        {"Pbm2", {{ItNumSetting(28,2)}}},
        {"Pbn2_1", {{ItNumSetting(33,2)}}},
        {"Pc11", {{ItNumSetting(7,6)}, {ItNumSetting(7,17)}}},
        {"Pc2_1b", {{ItNumSetting(29,5)}}},
        {"Pc2_1n", {{ItNumSetting(33,5)}}},
        {"Pc2a", {{ItNumSetting(32,5)}, {ItNumSetting(32,6)}}},
        {"Pc2m", {{ItNumSetting(28,5)}}},
        {"Pca2_1", {{ItNumSetting(29,1)}}},
        {"Pcc2", {{ItNumSetting(27,1)}, {ItNumSetting(27,2)}}},
        {"Pcm2_1", {{ItNumSetting(26,2)}}},
        {"Pcn2", {{ItNumSetting(30,2)}}},
        {"Pm11", {{ItNumSetting(6,5)}, {ItNumSetting(6,6)}}},
        {"Pm2_1b", {{ItNumSetting(26,6)}}},
        {"Pm2_1n", {{ItNumSetting(31,6)}}},
        {"Pm2a", {{ItNumSetting(28,6)}}},
        {"Pm2m", {{ItNumSetting(25,5)}, {ItNumSetting(25,6)}}},
        {"Pma2", {{ItNumSetting(28,1)}}},
        {"Pmc2_1", {{ItNumSetting(26,1)}}},
        {"Pmm2", {{ItNumSetting(25,1)}, {ItNumSetting(25,2)}}},
        {"Pmn2_1", {{ItNumSetting(31,1)}}},
        {"Pn11", {{ItNumSetting(7,11)}, {ItNumSetting(7,12)}}},
        {"Pn2_1a", {{ItNumSetting(33,6)}}},
        {"Pn2_1m", {{ItNumSetting(31,5)}}},
        {"Pn2b", {{ItNumSetting(30,6)}}},
        {"Pn2n", {{ItNumSetting(34,5)}, {ItNumSetting(34,6)}}},
        {"Pna2_1", {{ItNumSetting(33,1)}}},
        {"Pnc2", {{ItNumSetting(30,1)}}},
        {"Pnm2_1", {{ItNumSetting(31,2)}}},
        {"Pnn2", {{ItNumSetting(34,1)}, {ItNumSetting(34,2)}}},
        {"R-3", {{ItNumSetting(148,1)}, {ItNumSetting(148,2)}}},
        {"R-32/c", {{ItNumSetting(167,1)}, {ItNumSetting(167,2)}}},
        {"R-32/m", {{ItNumSetting(166,1)}, {ItNumSetting(166,2)}}},
        {"R3", {{ItNumSetting(146,1)}, {ItNumSetting(146,2)}}},
        {"R32", {{ItNumSetting(155,1)}, {ItNumSetting(155,2)}}},
        {"R3c", {{ItNumSetting(161,1)}, {ItNumSetting(161,2)}}},
        {"R3m", {{ItNumSetting(160,1)}, {ItNumSetting(160,2)}}}
    };

    void parseXml(const QDomDocument &);
    QString sgBgmnToCifFull(const QString &, bool, bool);
    QString sgBgmnToCifShort(const QString &, bool, bool);
    QList<BgmnSpaceGroup> checkForPerfectSGMatch(const QMap<int, BgmnSpaceGroup> &, const QString &);
    QMultiMap<QString, int> getAllBgmnHMSymbols(int);
    bool getSpaceGroupSettingNode(int it, int setting, QDomNode &);
    QList<QStringList> getSymmetryOperationsTranslated(int it, int setting);
};

#endif // BGMNSGDATPARSER_H
