#include <QApplication>
#include <QStringList>
#include <QString>
#include <QFile>
#include <QDir>
#include <QDateTime>
#include <QFileInfo>
#include <QStandardPaths>

#include "mainwindow.h"

#ifndef Q_OS_LINUX
    QTextStream *str = nullptr;

    QString logFileName()
    {
        QFileInfo logDir = QFileInfo(QStandardPaths::writableLocation(QStandardPaths::AppDataLocation) + "/");
        // if the default location is not writable, try some alternatives
        if (!logDir.isWritable()) {
            logDir = QFileInfo(QDir::homePath() + "/");
        }

        return logDir.absolutePath() + "/profex.log";
    }

    void myMessageOutput(QtMsgType type, const QMessageLogContext &context, const QString &msg)
    {
        Q_UNUSED(type);
        Q_UNUSED(context);

        QDateTime date;
        (*str) << QString("[%1] %2\n").arg(date.currentDateTime().toString()).arg(msg);
        str->flush();
    }
#endif

int main(int argc, char *argv[])
{
    QCoreApplication::setOrganizationName("doebelin.org");
    QCoreApplication::setApplicationName("Profex5");

#ifdef Q_OS_WIN
    QApplication::setStyle(QStyleFactory::create("Fusion"));
#endif

#ifdef Q_OS_LINUX
    QApplication::setStyle(QStyleFactory::create("Fusion"));
#endif

#ifdef Q_OS_MACOS
    QApplication::setStyle(QStyleFactory::create("macOS"));
#endif

    QApplication a(argc, argv);

    QString logDest("(not set)");

    // we only write the log file on Windows and OS X. On Linux we can easily get the debug output
    // in the console
    #ifdef QT_NO_DEBUG_OUTPUT
        logDest = "(debug output disabled)";
    #else
        #ifdef Q_OS_LINUX
            logDest = "Console";
        #else
            QFileInfo lfi(logFileName());
            QFile outFile(lfi.absoluteFilePath());
            outFile.remove();

            if (outFile.open(QIODevice::WriteOnly | QIODevice::Append | QIODevice::Text)) {
                logDest = lfi.absoluteFilePath();
                str = new QTextStream(&outFile);
                qInstallMessageHandler(myMessageOutput);
            } else {
                logDest = QString("Disabled (could not open file %1").arg(lfi.absoluteFilePath());
            }
        #endif
    #endif

    MainWindow w;
    QStringList l;

    for (int i = 1; i < argc; ++i) {
        l.append(QString(argv[i]));
    }

    if (l.size()) {
        w.loadFileList(l);
    }

    w.setLogDest(logDest);

    if (w.doShowMaximized()) {
        w.showMaximized();
    } else {
        w.show();
    }

    w.postShowInitialization();

    return a.exec();
}
