/***************************************************************************
                          chemtabledata.h  -  description
                             -------------------
    begin                : Sun Feb 21 13:41:07 CEST 2016
    copyright            : (C) 2016 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CHEMTABLEDATA_H
#define CHEMTABLEDATA_H

#include <QString>
#include <QStringList>
#include <QMap>
#include <QHash>
#include "../libXrdIO/settingsmanager.h"
#include "../libXrdIO/structs.h"
#include "../libXrdIO/crystal/crystalstructure.h"

using namespace global;

typedef QString Phase;

enum ChemistryMode {ELEMENT, ATOMIC, OXIDE};

class ChemTableData
{
public:
    ChemTableData();

    void clear();
    void readSettings();
    void setAtomsAndGoals(const QMultiMap<Phase, CrystalStructure> &phasesStructures);
    QStringList getAllPhaseNames();
    QStringList getAllOxideNames();
    QList<Oxide> getPhaseChemistry(const QString &);
    QHash<QString, double> getPhaseChemWeights(const QString &);
    QHash<QString, double> getPhaseChemEsds(const QString &);
    inline int getPhaseCount() {return phaseWeightData.size();}

private:
    SettingsManager *settings;
    ChemistryMode mode;
    QHash<QString, int> oxideNumberLookupTable;
    QHash<QString, Oxide> atomOxideLookupTable;
    QHash<QString, int> elementNumberLookupTable;
    QHash<QString, Oxide> atomElementLookupTable;
    QMultiMap<Phase, Oxide> phaseWeightData;

    void initOxideList(const QStringList &);
    void initElementList(const QStringList &);
    QString appendOxideToLUTs(const QStringList &);
    QStringList defaultOxideWeights();
    QStringList defaultElementWeights();
    QMultiMap<Phase, Oxide> structuresToAtoms(const QMultiMap<Phase, CrystalStructure> &);
    QList<Oxide> atomsToOxidesNormalized(const QList<CrystalAtom> &atomList);
    QList<Oxide> atomsToElementsNormalized(const QList<CrystalAtom> &atomList);
    QList<Oxide> atomsToAtomicNormalized(const QList<CrystalAtom> &atomList);
};

#endif // CHEMTABLEDATA_H
