/***************************************************************************
                          chemtablewidget.h  -  description
                             -------------------
    begin                : Mon Jun 30 14:16:07 CEST 2014
    copyright            : (C) 2014 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CHEMTABLEWIDGET_H
#define CHEMTABLEWIDGET_H

#include "../libXrdIO/structs.h"
#include "../libXrdIO/crystal/crystalstructure.h"
#include "../libXrdIO/parser/bgmnlstparser.h"
#include "chemtabledata.h"
#include "projectWidget/graphWidget/abstractgraphview.h"
#include <QTableWidget>
#include <QColor>
#include <QComboBox>
#include <QMouseEvent>
#include <QMenu>
#include <QAction>

using namespace global;

namespace Ui {
class ChemTableWidgetForm;
}

class ChemTableWidget : public AbstractGraphView
{
    Q_OBJECT
public:
    explicit ChemTableWidget(GraphDataController *, QWidget *parent = 0);
    ~ChemTableWidget();

    /* update the view when visibility changes */
    void setShown(bool);

    /* Reads data from an lst file and updates the table.
     * Normally the table is only updated when the widget is shown.
     * But when gathering chemtable data from all projects for export,
     * we need to force update even if the table is hidden. In that case,
     * set force = true.
     */
    void setData(const QString &lstFile, bool force = false);
    void setData(const BgmnLstParser &lparser, bool force = false);

    /* must be called after changing the settings, to apply changes
     * of the oxide / element format */
    void initSettings();

    /* clears the table */
    void clearAll();

    /* return true if the table contains valid data */
    bool hasData();

    /* return results in different formats */
    QStringList getCsvList(bool header = false, const QString &sampleId = QString()); // List with one parameter per line
    QString getCsvTable();                   // Table as shown on screen
    QString getHtmlTable();                  // Table as shown on screen

public slots:
    void updateView();

private:
    Ui::ChemTableWidgetForm *ui;
    ChemistryMode mode;
    ChemTableData oxideData;
    QString fileName;
    QString sampleName;
    QHash<QString, QVariant> previousManualGoalAssignments;
    QMenu *contextMenu;

    bool reload();
    bool cellIsValid(int, int);
    void resizeTable();
    void createPhaseComboBoxes(const QList<Result> &);
    void calculateQuantityCells();
    void calculatePhaseCells();
    void calculateTotalCells();
    void clearTotalCells();
    bool allQuantityGoalsSelected();
    double getPhaseQuantity(int);
    void setCellValue(int, int, double, double, const QColor &color = QColor(), int digits = 2);
    void setCellText(int, int, const QString &, const QColor &color = QColor());
    double getCellValue(int row, int column, double defaultValue = 0.0);
    double getCellEsd(int row, int column, double defaultValue = 0.0);
    bool selectComboBoxItem(QComboBox *, const QString &);
    void updateManualGoalAssignments();
    QString getQuantityGoal(int) const;
    QString getQuantityValue(int) const;
    QMap<int, double> normalizeGoalValues();

    void mousePressEvent(QMouseEvent *);

private slots:
    void recalculate();
    void copy();
    void headerResized();
};

#endif // CHEMTABLEWIDGET_H
