/***************************************************************************
                          projectsearchmatchwidget.h  -  description
                             -------------------
    begin                : Tue Jan 22 21:00:00 CEST 2019
    copyright            : (C) 2019 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PEAKLISTWIDGET_H
#define PEAKLISTWIDGET_H

#include <QWidget>
#include <QTreeWidgetItem>
#include <QStandardItemModel>
#include <QDomElement>
#include "../graphWidget/abstractgraphview.h"
#include "../bgmnrefstructuremanager.h"
#include "hklsortfilterproxymodel.h"

namespace Ui {
class PeakListWidget;
}

class PeakListWidget : public AbstractGraphView
{
    Q_OBJECT

public:
    explicit PeakListWidget(GraphDataController *c, QWidget *parent = nullptr);
    ~PeakListWidget();

    void clearAll();

    QString getCsvData(bool header) const;
    QString getQualxDdata() const;
    QString getQualxTdata() const;

    QDomElement getFilterParameters() const;
    void setFilterParameters(const QDomElement &);

public slots:
    void updateView();
    void updateData(bool force = false);

private:
    Ui::PeakListWidget *ui;
    BgmnRefStructureManager *refStrManager;
    QStringList hklHeaderLabels;
    QStandardItemModel *peakDataModel;
    HklSortFilterProxyModel *peakDataProxy;
    QHash<QString, int> columns;
    QMenu *contextMenuHeader;
    QMenu *toolsMenu;
    bool saveSettingsRequested;
    int prevNumberOfScans;
    QList<int> previousSplitterSizes;

    void initSettings();

    void appendDataToModel(const Scan *, double, double wl);

    void saveToCsv(const QString &);
    void saveToQualxD(const QString &);
    void saveToQualxT(const QString &);
    QList<int> hklToInt(const QString &);
    int indexOfHkl(const Scan *, const QUuid &);

private slots:
    void resultSelected(QTreeWidgetItem *);
    void saveHklList();
    void clearHklSelection();
    void selectHkl(const QItemSelection &, const QItemSelection &);
    void removeCurrentHkl();
    void openCustomContextMenu(const QPoint &);
    void toggleColumnHidden(QAction *);
    void headerResized();
    void splitterResized(int, int);
    void phaseFilterChanged(int);
    void toggleFilterWidget();
    void showHelp();

signals:
    void resultsSelectionChanged(const QString &);
    void helpText(QString);
    void applyFiltersToAll();
};

#endif // PEAKLISTWIDGET_H
