/***************************************************************************
                          baselinedialog.h  -  description
                             -------------------
    begin                : Tue Jan 23 22:00:00 CEST 2018
    copyright            : (C) 2018 by Nicola Doebelin
    email                : ndoebelin@gmx.ch
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef SCANSMOOTHDIALOG_H
#define SCANSMOOTHDIALOG_H

#include <QDialog>
#include <QVector>
#include <QTableWidgetItem>

#include "tools/abstracttooldialog.h"
#include "projectWidget/graphWidget/graphwindow.h"

#include "../libXrdIO/settingsmanager.h"
#include "../libXrdIO/scan.h"
#include "../libXrdIO/scanops.h"

namespace Ui {
class ScanSmoothDialog;
}

const QString smAuxString = "isSmoothedScan";
enum SmoothAlgoType {SMA, EMA, LRMA};

class ScanSmoothDialog : public AbstractToolDialog
{
    Q_OBJECT

public:
    explicit ScanSmoothDialog(QWidget *parent = 0);
    ~ScanSmoothDialog();

    void clearTemporary();

private:
    Ui::ScanSmoothDialog *ui;
    QStringList headers;

    void preSetProject(ProjectWidget *);
    void postSetProject(ProjectWidget *);
    void clearGui();
    void initSettings();
    void saveSettings();
    void parseScans();
    void keepTemporary();
    void blockUpdateSignals(bool);
    void computeAlgo(SmoothAlgoType);

private slots:
    void updateView();
    void algoChanged(int);
    void compute();
    void computeSma();
    void computeEma();
    void computeLrma();
    void append();
    void resetSma();
    void resetEma();
    void resetLrma();
};

#endif // SCANSMOOTHDIALOG_H
